# All Rights Reserved 2018
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from osc_lib.cli import format_columns
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils as osc_utils
from osc_lib.utils import columns as column_util

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.osc import utils as nc_osc_utils

LOG = logging.getLogger(__name__)

INTERFACE_DELIMITER = ";"
SEGMENTATION_ID_DELIMITER = "#"
INTERFACE_SEG_ID_DELIMITER = "|"

L2_GATEWAY = 'l2_gateway'
L2_GATEWAYS = '%ss' % L2_GATEWAY

path = 'l2-gateways'
object_path = '/%s' % path
resource_path = '/%s/%%s' % path

_attr_map = (
    ('id', 'ID', column_util.LIST_BOTH),
    ('tenant_id', 'Tenant', column_util.LIST_LONG_ONLY),
    ('name', 'Name', column_util.LIST_BOTH),
    ('devices', 'Devices', column_util.LIST_BOTH),
)
_formatters = {
    'devices': format_columns.ListDictColumn,
}


def _get_common_parser(parser):
    """Adds to parser arguments common to create and update commands.

    :params ArgumentParser parser: argparse object contains all command's
                                   arguments
    """
    parser.add_argument(
        '--device',
        metavar='name=name,interface_names=INTERFACE-DETAILS',
        action='append', dest='devices', type=utils.str2dict,
        help=_('Device name and Interface-names of l2gateway. '
               'INTERFACE-DETAILS is of form '
               '\"<interface_name1>;[<interface_name2>]'
               '[|<seg_id1>[#<seg_id2>]]\" '
               '(--device option can be repeated)'))


def get_interface(interfaces):
    interface_dict = []
    for interface in interfaces:
        if INTERFACE_SEG_ID_DELIMITER in interface:
            int_name = interface.split(INTERFACE_SEG_ID_DELIMITER)[0]
            segid = interface.split(INTERFACE_SEG_ID_DELIMITER)[1]
            if SEGMENTATION_ID_DELIMITER in segid:
                segid = segid.split(SEGMENTATION_ID_DELIMITER)
            else:
                segid = [segid]
            interface_detail = {'name': int_name, 'segmentation_id': segid}
        else:
            interface_detail = {'name': interface}
        interface_dict.append(interface_detail)
    return interface_dict


def _args2body(parsed_args, update=False):
    if parsed_args.devices:
        devices = parsed_args.devices
        interfaces = []
    else:
        devices = []
    device_dict = []
    for device in devices:
        if 'interface_names' in device.keys():
            interface = device['interface_names']
            if INTERFACE_DELIMITER in interface:
                interface_dict = interface.split(INTERFACE_DELIMITER)
                interfaces = get_interface(interface_dict)
            else:
                interfaces = get_interface([interface])
        if 'name' in device.keys():
            device = {'device_name': device['name'],
                      'interfaces': interfaces}
        else:
            device = {'interfaces': interfaces}
        device_dict.append(device)
    if parsed_args.name:
        l2gw_name = parsed_args.name
        body = {L2_GATEWAY: {'name': l2gw_name,
                             'devices': device_dict}, }
    else:
        body = {L2_GATEWAY: {'devices': device_dict}, }
    return body


class CreateL2gw(command.ShowOne):
    _description = _("Create l2gateway resource")

    def get_parser(self, prog_name):
        parser = super(CreateL2gw, self).get_parser(prog_name)
        nc_osc_utils.add_project_owner_option_to_parser(parser)
        parser.add_argument(
            'name', metavar='<GATEWAY-NAME>',
            help=_('Descriptive name for logical gateway.'))
        _get_common_parser(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = {}
        if parsed_args.name is not None:
            attrs['name'] = str(parsed_args.name)
        if parsed_args.devices is not None:
            attrs['devices'] = str(parsed_args.devices)
        if 'project' in parsed_args and parsed_args.project is not None:
            project_id = nc_osc_utils.find_project(
                self.app.client_manager.identity,
                parsed_args.project,
                parsed_args.project_domain,
            ).id
            attrs['tenant_id'] = project_id
        body = _args2body(parsed_args)
        obj = client.post(object_path, body)[L2_GATEWAY]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = osc_utils.get_dict_properties(obj, columns,
                                             formatters=_formatters)
        return display_columns, data


class ListL2gw(command.Lister):
    _description = _("List l2gateway that belongs to a given tenant")

    def get_parser(self, prog_name):
        parser = super(ListL2gw, self).get_parser(prog_name)
        nc_osc_utils.add_project_owner_option_to_parser(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        params = {}
        if parsed_args.project is not None:
            project_id = nc_osc_utils.find_project(
                self.app.client_manager.identity,
                parsed_args.project,
                parsed_args.project_domain,
            ).id
            params['tenant_id'] = project_id
        objs = client.list(L2_GATEWAYS, object_path,
                           retrieve_all=True, params=params)[L2_GATEWAYS]
        headers, columns = column_util.get_column_definitions(
            _attr_map, long_listing=True)
        return (headers, (osc_utils.get_dict_properties(
            s, columns, formatters=_formatters) for s in objs))


class ShowL2gw(command.ShowOne):
    _description = _("Show information of a given l2gateway")

    def get_parser(self, prog_name):
        parser = super(ShowL2gw, self).get_parser(prog_name)
        parser.add_argument(
            L2_GATEWAY,
            metavar="<L2_GATEWAY>",
            help=_("ID or name of l2_gateway to look up."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = client.find_resource(L2_GATEWAY, parsed_args.l2_gateway)['id']
        obj = client.get(resource_path % id)[L2_GATEWAY]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = osc_utils.get_dict_properties(obj, columns,
                                             formatters=_formatters)
        return display_columns, data


class DeleteL2gw(command.Command):
    _description = _("Delete a given l2gateway")

    def get_parser(self, prog_name):
        parser = super(DeleteL2gw, self).get_parser(prog_name)
        parser.add_argument(
            L2_GATEWAYS,
            metavar="<L2_GATEWAY>",
            nargs="+",
            help=_("ID(s) or name(s) of l2_gateway to delete."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        fails = 0
        for id_or_name in parsed_args.l2_gateways:
            try:
                id = client.find_resource(L2_GATEWAY, id_or_name)['id']
                client.delete(resource_path % id)
                LOG.warning("L2 Gateaway %(id)s deleted", {'id': id})
            except Exception as e:
                fails += 1
                LOG.error("Failed to delete L2 Gateway with name or ID "
                          "'%(id_or_name)s': %(e)s",
                          {'id_or_name': id_or_name, 'e': e})
        if fails > 0:
            msg = (_("Failed to delete %(fails)s of %(total)s L2 Gateway.") %
                   {'fails': fails, 'total': len(parsed_args.l2_gateways)})
            raise exceptions.CommandError(msg)


class UpdateL2gw(command.ShowOne):
    _description = _("Update a given l2gateway")

    def get_parser(self, prog_name):
        parser = super(UpdateL2gw, self).get_parser(prog_name)
        parser.add_argument(
            L2_GATEWAY,
            metavar="<L2_GATEWAY>",
            help=_("ID or name of l2_gateway to update."),
        )
        parser.add_argument('--name', metavar='name',
                            help=_('Descriptive name for logical gateway.'))
        _get_common_parser(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = client.find_resource(L2_GATEWAY, parsed_args.l2_gateway)['id']
        if parsed_args.devices:
            body = _args2body(parsed_args)
        else:
            body = {L2_GATEWAY: {'name': parsed_args.name}}
        obj = client.put(resource_path % id, body)[L2_GATEWAY]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = osc_utils.get_dict_properties(obj, columns,
                                             formatters=_formatters)
        return display_columns, data



# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutron.tests import base
from neutron_lib import rpc as n_rpc
from oslo_config import cfg
import ovs.jsonrpc
from ovsdbapp.backend.ovs_idl import command as cmd
from ovsdbapp.backend.ovs_idl import idlutils

from networking_l2gw.services.l2gateway.agent.ovsdb import impl_idl


class Msg(object):
    id = 0
    type = 1
    method = 'update'
    params = [
        None, {'Global': {
            '381d60b5-6171-484a-842f-49f3e83bb586': {'new': {
                'switches': ['uuid', '03eebc5d-153f-4b03-8efa-45d22eb9942f'],
                'other_config': [
                    'map', []
                ],
                'managers': ['set', []]}
            }},
            'Physical_Switch': {'03eebc5d-153f-4b03-8efa-45d22eb9942f': {
                'new': {'management_ips': ['set', []],
                        'description': '', 'other_config': ['map', []],
                        'tunnel_ips': ['set', []],
                        'switch_fault_status': ['set', []],
                        'ports': ['set', []], 'tunnels': ['set', []],
                        'name': 'ps1'}}}}
    ]


class SimpleIdlTests(base.BaseTestCase):
    def setUp(self):
        super(SimpleIdlTests, self).setUp()
        n_rpc.init(cfg.CONF)

    def test_list_physical_switches(self):
        session_mock = mock.patch.object(
            ovs.jsonrpc.Session,
            'open',
            return_value=ovs.jsonrpc.Session(None, None, None)
        )
        wait_mock = mock.patch.object(idlutils, 'wait_for_change')

        execute_mock = mock.patch.object(
            cmd.BaseCommand,
            'execute',
            return_value=['ps1'])
        with session_mock, execute_mock, wait_mock:
            ovs.jsonrpc.Session.run = mock.MagicMock(
                return_value=mock.MagicMock(return_value='myrun'))
            ovs.jsonrpc.Session.get_seqno = mock.MagicMock(return_value=None)
            ovs.jsonrpc.Session.wait = mock.MagicMock(return_value=None)
            ovs.jsonrpc.Session.is_connected = mock.MagicMock(
                return_value=True)
            ovs.jsonrpc.Session.recv = mock.MagicMock(
                side_effect=[Msg, None, None, None, None])

            self.db = impl_idl.OvsdbHardwareVtepIdl(self,
                                                    'tcp:fake_ip:fake_port',
                                                    3)

            sw_list = self.db.get_physical_sw_list().execute()

            assert (sw_list is not None) & (sw_list[0] == 'ps1')
            print('physical switch: %s' % sw_list)



# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from neutron.common import config as common_config
from neutron.conf.agent import common as agent_config
from neutron_lib import rpc as n_rpc
from oslo_config import cfg
from oslo_service import service

from networking_l2gw.services.l2gateway.agent.ovsdb import manager
from networking_l2gw.services.l2gateway.common import config
from networking_l2gw.services.l2gateway.common import topics


class L2gatewayAgentService(n_rpc.Service):
    def start(self):
        super(L2gatewayAgentService, self).start()


def main():
    common_config.register_common_config_options()
    config.register_ovsdb_opts_helper(cfg.CONF)
    agent_config.register_agent_state_opts_helper(cfg.CONF)
    common_config.init(sys.argv[1:])
    config.setup_logging()

    mgr = manager.OVSDBManager(cfg.CONF)
    svc = L2gatewayAgentService(
        host=cfg.CONF.host,
        topic=topics.L2GATEWAY_AGENT,
        manager=mgr
    )
    service.launch(cfg.CONF, svc).wait()



# Copyright 2015 OpenStack Foundation
# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutron_lib.api import validators
from neutron_lib import exceptions

from networking_l2gw._i18n import _
from networking_l2gw.services.l2gateway.common import constants

ALLOWED_CONNECTION_ATTRIBUTES = set((constants.NETWORK_ID,
                                     constants.SEG_ID,
                                     constants.L2GATEWAY_ID
                                     ))


def validate_gwdevice_list(data, valid_values=None):
    """Validate the list of devices."""
    if not data:
        # Devices must be provided
        msg = _("Cannot create a gateway with an empty device list")
        return msg
    try:
        for device in data:
            interface_data = device.get(constants.IFACE_NAME_ATTR)
            device_name = device.get(constants.DEVICE_ID_ATTR)
            if not device_name:
                msg = _("Cannot create a gateway with an empty device_name")
                return msg
            if not interface_data:
                msg = _("Cannot create a gateway with an empty interfaces")
                return msg
            if not isinstance(interface_data, list):
                msg = _("interfaces format is not a type list of dicts")
                return msg
            for int_dict in interface_data:
                if not isinstance(int_dict, dict):
                    msg = _("interfaces format is not a type dict")
                    return msg
                err_msg = validators.validate_dict(int_dict, None)
                if not int_dict.get('name'):
                    msg = _("Cannot create a gateway with an empty "
                            "interface name")
                    return msg
                if constants.SEG_ID in int_dict:
                    seg_id_list = int_dict.get(constants.SEG_ID)
                    if seg_id_list and type(seg_id_list) is not list:
                        msg = _("segmentation_id type should be of list type ")
                        return msg
                    if not seg_id_list:
                        msg = _("segmentation_id_list should not be empty")
                        return msg
                    for seg_id in seg_id_list:
                        is_valid_vlan_id(seg_id)
                    if err_msg:
                        return err_msg
    except TypeError:
        return (_("%s: provided data are not iterable") %
                validate_gwdevice_list.__name__)


def validate_network_mapping_list(network_mapping, check_vlan):
    """Validate network mapping list in connection."""
    if network_mapping.get('segmentation_id'):
        if check_vlan:
            raise exceptions.InvalidInput(
                error_message=_("default segmentation_id should not be"
                                " provided when segmentation_id is assigned"
                                " during l2gateway creation"))
        seg_id = network_mapping.get(constants.SEG_ID)
        is_valid_vlan_id(seg_id)

    if not network_mapping.get('segmentation_id'):
        if check_vlan is False:
            raise exceptions.InvalidInput(
                error_message=_("Segmentation id must be specified in create "
                                "l2gateway connections"))
    network_id = network_mapping.get(constants.NETWORK_ID)
    if not network_id:
        raise exceptions.InvalidInput(
            error_message=_("A valid network identifier must be specified "
                            "when connecting a network to a network "
                            "gateway. Unable to complete operation"))
    connection_attrs = set(network_mapping.keys())
    if not connection_attrs.issubset(ALLOWED_CONNECTION_ATTRIBUTES):
        raise exceptions.InvalidInput(
            error_message=(_("Invalid keys found among the ones provided "
                             "in request : %(connection_attrs)s."),
                           connection_attrs))
    return network_id


def is_valid_vlan_id(seg_id):
    try:
        int_seg_id = int(seg_id)
    except ValueError:
        msg = _("Segmentation id must be a valid integer")
        raise exceptions.InvalidInput(error_message=msg)
    if int_seg_id < 0 or int_seg_id >= 4095:
        msg = _("Segmentation id is out of range")
        raise exceptions.InvalidInput(error_message=msg)



# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutron.agent import rpc as agent_rpc
from neutron.conf.agent import common as agent_config
from neutron.tests import base

from oslo_config import cfg
from oslo_service import loopingcall

from networking_l2gw.services.l2gateway.agent import (base_agent_manager
                                                      as l2gw_manager)
from networking_l2gw.services.l2gateway.agent import agent_api
from networking_l2gw.services.l2gateway.common import constants as n_const


class TestBaseAgentManager(base.BaseTestCase):

    def setUp(self):
        super(TestBaseAgentManager, self).setUp()
        agent_config.register_agent_state_opts_helper(cfg.CONF)
        cfg.CONF.set_override('report_interval', 1, 'AGENT')
        self.context = mock.Mock
        mock.patch('neutron.agent.rpc.PluginReportStateAPI').start()
        self.l2gw_agent_manager = l2gw_manager.BaseAgentManager(
            cfg.CONF)

    def test_init(self):
        with mock.patch.object(agent_api,
                               'L2GatewayAgentApi') as l2_gw_agent_api:
            with mock.patch.object(l2gw_manager.BaseAgentManager,
                                   '_setup_state_rpc') as setup_state_rpc:
                self.l2gw_agent_manager.__init__(mock.Mock())
                self.assertEqual('',
                                 self.l2gw_agent_manager.l2gw_agent_type)
                self.assertTrue(self.l2gw_agent_manager.admin_state_up)
                self.assertTrue(setup_state_rpc.called)
                self.assertTrue(l2_gw_agent_api.called)

    def test_setup_state_rpc(self):
        cfg.CONF.set_override('report_interval', 1, 'AGENT')
        with mock.patch.object(agent_rpc,
                               'PluginReportStateAPI'
                               ) as agent_report_state_rpc:
            with mock.patch.object(loopingcall,
                                   'FixedIntervalLoopingCall'
                                   ) as looping_call:
                self.l2gw_agent_manager._setup_state_rpc()
                self.assertTrue(agent_report_state_rpc.called)
                self.assertTrue(looping_call.called)
                looping_call.return_value.start.assert_called_with(
                    interval=mock.ANY)

    def test_report_state(self):
        with mock.patch.object(self.l2gw_agent_manager,
                               'state_rpc') as state_api:
            self.assertTrue(self.l2gw_agent_manager.agent_state['start_flag'])
            self.l2gw_agent_manager._report_state()
            self.assertFalse(self.l2gw_agent_manager.agent_state['start_flag'])
            self.assertTrue(state_api.report_state.called)

    def test_report_state_exception(self):
        cfg.CONF.set_override('report_interval', 1, 'AGENT')
        with mock.patch.object(self.l2gw_agent_manager,
                               'state_rpc') as state_rpc:
            with mock.patch.object(l2gw_manager.LOG, 'exception') as exc:
                with mock.patch.object(self.l2gw_agent_manager,
                                       'handle_report_state_failure'
                                       ) as mock_handle_report_state_failure:
                    state_rpc.report_state.side_effect = Exception()
                    self.l2gw_agent_manager._report_state()
                    self.assertTrue(exc.called)
                    self.assertTrue(mock_handle_report_state_failure.called)

    def test_agent_updated(self):
        fake_payload = {'fake_key': 'fake_value'}
        with mock.patch.object(l2gw_manager.LOG, 'info') as logger_call:
            self.l2gw_agent_manager.agent_updated(mock.Mock(), fake_payload)
            self.assertEqual(1, logger_call.call_count)

    def test_set_monitor_agent_type_monitor(self):
        self.l2gw_agent_manager.l2gw_agent_type = ''
        self.l2gw_agent_manager.conf.host = 'fake_host'
        self.l2gw_agent_manager.set_monitor_agent(self.context, 'fake_host')
        self.assertEqual(n_const.MONITOR,
                         self.l2gw_agent_manager.agent_state.
                         get('configurations')[n_const.L2GW_AGENT_TYPE])
        self.assertEqual(n_const.MONITOR,
                         self.l2gw_agent_manager.l2gw_agent_type)

    def test_set_monitor_agent_type_transact(self):
        self.l2gw_agent_manager.l2gw_agent_type = ''
        cfg.CONF.host = 'fake_host'
        self.l2gw_agent_manager.set_monitor_agent(
            self.context, 'fake_host1')
        self.assertNotEqual(n_const.MONITOR,
                            self.l2gw_agent_manager.agent_state.
                            get('configurations')[n_const.L2GW_AGENT_TYPE])
        self.assertEqual('',
                         self.l2gw_agent_manager.agent_state.
                         get('configurations')[n_const.L2GW_AGENT_TYPE])
        self.assertEqual('',
                         self.l2gw_agent_manager.l2gw_agent_type)



# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random
import socket
import ssl
from unittest import mock


from neutron.tests import base

from networking_l2gw.services.l2gateway.agent import l2gateway_config as conf
from networking_l2gw.services.l2gateway.agent.ovsdb import base_connection
from networking_l2gw.services.l2gateway.agent.ovsdb import ovsdb_writer
from networking_l2gw.services.l2gateway.common import config
from networking_l2gw.services.l2gateway.common import constants as n_const
from networking_l2gw.services.l2gateway.common import ovsdb_schema
from networking_l2gw.services.l2gateway import exceptions
from networking_l2gw.tests.unit.services.l2gateway.agent.ovsdb import (
    test_base_connection as base_test)

from oslo_config import cfg
from oslo_log import log as logging
from oslo_serialization import jsonutils

LOG = logging.getLogger(__name__)


class TestOVSDBWriter(base.BaseTestCase):
    def setUp(self):
        super(TestOVSDBWriter, self).setUp()
        self.conf = mock.patch.object(conf, 'L2GatewayConfig').start()
        config.register_ovsdb_opts_helper(cfg.CONF)
        cfg.CONF.set_override('max_connection_retries', 0, 'ovsdb')

        self.sock = mock.patch('socket.socket').start()
        self.ssl_sock = mock.patch.object(ssl, 'wrap_socket').start()
        self.op_id = 'abcd'
        self.l2gw_ovsdb = ovsdb_writer.OVSDBWriter(mock.Mock(),
                                                   self.conf)
        self.fake_message = {'id': self.op_id,
                             'fake_key': 'fake_value'}

        self.l2gw_ovsdb.responses = [self.fake_message]

    def test_process_response(self):
        """Test case to test _process_response."""
        expected_result = {'fake_key': 'fake_value'}
        with mock.patch.object(base_connection.BaseConnection,
                               '_response',
                               return_value={'fake_key': 'fake_value'}
                               ) as resp:
            result = self.l2gw_ovsdb._process_response(self.op_id)
            self.assertEqual(result, expected_result)
            resp.assert_called_with(self.op_id)

    def test_process_response_with_error(self):
        """Test case to test _process_response with error."""
        foo_dict = {'fake_key': 'fake_value',
                    'error': 'fake_error'}
        with mock.patch.object(base_connection.BaseConnection,
                               '_response',
                               return_value=foo_dict) as resp:
            self.assertRaises(exceptions.OVSDBError,
                              self.l2gw_ovsdb._process_response,
                              self.op_id)
            resp.assert_called_with(self.op_id)

    def test_process_response_with_error1(self):
        """Test case to test _process_response with errors in the

           subqueries.
        """
        fake_dict = {"id": "295366252499790541931626006259650283530",
                     "result":
                     [{"uuid":
                       ["uuid", "be236bbf-8f83-4bf0-816b-629c7e5b5609"
                        ]},
                      {},
                      {"error": "referential integrity violation",
                       "details": "Table Ucast_Macs_Remote column "
                                  "locator row "
                                  "be236bbf-8f83-4bf0-816b-629c7e5b5609 "
                                  "references nonexistent row "
                                  "1b143819-45a6-44ec-826a-ac75243a07ce in "
                                  "table Physical_Locator."
                       }],
                     "error": None}
        with mock.patch.object(base_connection.BaseConnection,
                               '_response',
                               return_value=fake_dict) as resp:
            self.assertRaises(exceptions.OVSDBError,
                              self.l2gw_ovsdb._process_response,
                              self.op_id)
            resp.assert_called_with(self.op_id)

    def test_get_reply(self):
        """Test case to test _get_reply."""
        ret_value = jsonutils.dumps({self.op_id:
                                     'foo_value'})
        with mock.patch.object(
            ovsdb_writer.OVSDBWriter,
            '_recv_data',
            return_value=jsonutils.dumps({self.op_id:
                                          'foo_value'})) as recv_data, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              '_process_response',
                              return_value=(ret_value, None)
                              ) as proc_response, \
            mock.patch.object(ovsdb_writer.LOG,
                              'debug'):
            self.l2gw_ovsdb._get_reply(self.op_id, mock.ANY)
            self.assertTrue(recv_data.called)
            self.assertTrue(proc_response.called)

    def test_send_and_receive(self):
        """Test case to test _send_and_receive."""
        with mock.patch.object(base_connection.BaseConnection,
                               'send', return_value=True
                               ) as mock_send:
            with mock.patch.object(ovsdb_writer.OVSDBWriter,
                                   '_get_reply') as mock_reply:
                self.l2gw_ovsdb._send_and_receive('some_query',
                                                  self.op_id, mock.ANY, True)
                mock_send.assert_called_with('some_query', addr=mock.ANY)
                mock_reply.assert_called_with(self.op_id, mock.ANY)

    def test_send_and_receive_with_rcv_required_false(self):
        """Test case to test _send_and_receive."""
        with mock.patch.object(base_connection.BaseConnection,
                               'send', return_value=True
                               ) as mock_send:
            with mock.patch.object(ovsdb_writer.OVSDBWriter,
                                   '_get_reply') as mock_reply:
                self.l2gw_ovsdb._send_and_receive('some_query',
                                                  self.op_id, mock.ANY, False)
                mock_send.assert_called_with('some_query', addr=mock.ANY)
                mock_reply.assert_not_called()

    def test_delete_logical_switch(self):
        """Test case to test delete_logical_switch."""
        commit_dict = {"op": "commit", "durable": True}
        query = {"method": "transact",
                 "params": [n_const.OVSDB_SCHEMA_NAME,
                            {"op": "delete",
                             "table": "Logical_Switch",
                             "where": [["_uuid", "==",
                                        ["uuid",
                                         'fake_logical_switch_uuid']]]},
                            commit_dict],
                 "id": self.op_id}
        with mock.patch.object(random,
                               'getrandbits',
                               return_value=self.op_id
                               ) as get_rand:
            with mock.patch.object(ovsdb_writer.OVSDBWriter,
                                   '_send_and_receive'
                                   ) as send_n_receive:
                with mock.patch.object(ovsdb_writer.LOG,
                                       'debug'):
                    self.l2gw_ovsdb.delete_logical_switch(
                        'fake_logical_switch_uuid', mock.ANY)
                    get_rand.assert_called_with(128)
                    send_n_receive.assert_called_with(query, self.op_id,
                                                      mock.ANY, True)

    def test_insert_ucast_macs_remote(self):
        """Test case to test insert ucast_macs_remote."""
        with mock.patch.object(ovsdb_writer.OVSDBWriter,
                               '_get_ucast_macs_remote_dict'
                               ) as get_ucast_mac_remote, \
                mock.patch.object(random,
                                  'getrandbits',
                                  return_value=self.op_id
                                  ) as get_rand, \
                mock.patch.object(ovsdb_writer.OVSDBWriter,
                                  '_send_and_receive'
                                  ) as send_n_receive, \
                mock.patch.object(ovsdb_writer.LOG,
                                  'debug'), \
                mock.patch.object(ovsdb_schema, 'LogicalSwitch'), \
                mock.patch.object(ovsdb_schema, 'PhysicalLocator'), \
                mock.patch.object(ovsdb_schema, 'UcastMacsRemote'):
            self.l2gw_ovsdb.insert_ucast_macs_remote(mock.MagicMock(),
                                                     mock.MagicMock(),
                                                     mock.MagicMock(),
                                                     mock.ANY)
            get_rand.assert_called_with(128)
            send_n_receive.assert_called_with(mock.ANY,
                                              self.op_id, mock.ANY, True)

            self.assertTrue(get_ucast_mac_remote.called)

    def test_insert_ucast_macs_remote_with_no_locator_id(self):
        """Test case to test insert ucast_macs_remote

           without locator_id and logical_switch_id.
        """
        with mock.patch.object(ovsdb_writer.OVSDBWriter,
                               '_get_ucast_macs_remote_dict'
                               ) as get_ucast_mac_remote, \
                mock.patch.object(ovsdb_writer.OVSDBWriter,
                                  '_get_physical_locator_dict'
                                  ) as get_physical_locator_dict, \
                mock.patch.object(ovsdb_writer.OVSDBWriter,
                                  '_get_logical_switch_dict'
                                  ) as get_logical_switch_dict, \
                mock.patch.object(random,
                                  'getrandbits',
                                  return_value=self.op_id
                                  ), \
                mock.patch.object(ovsdb_writer.OVSDBWriter,
                                  '_send_and_receive'
                                  ), \
                mock.patch.object(ovsdb_writer.LOG,
                                  'debug'), \
                mock.patch.object(ovsdb_schema,
                                  'LogicalSwitch') as mock_ls, \
                mock.patch.object(ovsdb_schema,
                                  'PhysicalLocator') as mock_pl, \
                mock.patch.object(ovsdb_schema,
                                  'UcastMacsRemote'):
            locator = mock_pl.return_value
            locator.uuid = None
            ls = mock_ls.return_value
            ls.uuid = None
            ls.name = 'ab-cd'
            self.l2gw_ovsdb.insert_ucast_macs_remote(mock.MagicMock(),
                                                     mock.MagicMock(),
                                                     mock.MagicMock(),
                                                     mock.ANY)
            self.assertTrue(get_ucast_mac_remote.called)
            self.assertTrue(get_physical_locator_dict.called)
            self.assertTrue(get_logical_switch_dict.called)

    def test_update_ucast_macs_remote(self):
        """Test case to test update ucast_macs_remote."""
        with mock.patch.object(ovsdb_writer.OVSDBWriter,
                               '_get_dict_for_update_ucast_mac_remote'
                               ) as get_update_ucast_mac_remote, \
                mock.patch.object(random,
                                  'getrandbits',
                                  return_value=self.op_id
                                  ) as get_rand, \
                mock.patch.object(ovsdb_writer.OVSDBWriter,
                                  '_send_and_receive'
                                  ) as send_n_receive, \
                mock.patch.object(ovsdb_writer.LOG,
                                  'debug'), \
                mock.patch.object(ovsdb_schema, 'PhysicalLocator'), \
                mock.patch.object(ovsdb_schema, 'UcastMacsRemote'):
            self.l2gw_ovsdb.update_ucast_macs_remote(mock.MagicMock(),
                                                     mock.MagicMock(),
                                                     mock.ANY)
            get_rand.assert_called_with(128)
            send_n_receive.assert_called_with(mock.ANY,
                                              self.op_id, mock.ANY, True)

            self.assertTrue(get_update_ucast_mac_remote.called)

    def test_update_ucast_macs_remote_with_no_locator_id(self):
        """Test case to test update ucast_macs_remote

           without locator_id and logical_switch_id.
        """
        with mock.patch.object(ovsdb_writer.OVSDBWriter,
                               '_get_dict_for_update_ucast_mac_remote'
                               ) as get_update_ucast_mac_remote, \
                mock.patch.object(ovsdb_writer.OVSDBWriter,
                                  '_get_physical_locator_dict'
                                  ) as get_physical_locator_dict, \
                mock.patch.object(random,
                                  'getrandbits',
                                  return_value=self.op_id
                                  ), \
                mock.patch.object(ovsdb_writer.OVSDBWriter,
                                  '_send_and_receive'
                                  ), \
                mock.patch.object(ovsdb_writer.LOG,
                                  'debug'), \
                mock.patch.object(ovsdb_schema,
                                  'PhysicalLocator') as mock_pl, \
                mock.patch.object(ovsdb_schema, 'UcastMacsRemote'):
            locator = mock_pl.return_value
            locator.uuid = None
            self.l2gw_ovsdb.update_ucast_macs_remote(mock.MagicMock(),
                                                     mock.MagicMock(),
                                                     mock.ANY)
            self.assertTrue(get_update_ucast_mac_remote.called)
            self.assertTrue(get_physical_locator_dict.called)

    def test_delete_ucast_macs_remote(self):
        """Test case to test delete_ucast_macs_remote."""
        with mock.patch.object(random,
                               'getrandbits',
                               return_value=self.op_id
                               ) as get_rand, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              '_send_and_receive'
                              ) as send_n_receive, \
            mock.patch.object(ovsdb_writer.LOG,
                              'debug'):
            self.l2gw_ovsdb.delete_ucast_macs_remote(mock.Mock(),
                                                     mock.MagicMock(),
                                                     mock.ANY)
            get_rand.assert_called_with(128)
            send_n_receive.assert_called_with(mock.ANY,
                                              self.op_id, mock.ANY, True)

    def test_update_connection_to_gateway(self):
        """Test case to test update_connection_to_gateway."""
        with mock.patch.object(ovsdb_writer.OVSDBWriter,
                               '_get_bindings_to_update') as get_bindings, \
            mock.patch.object(random,
                              'getrandbits',
                              return_value=self.op_id
                              ) as get_rand, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              '_send_and_receive'
                              ) as send_n_receive, \
            mock.patch.object(ovsdb_writer.LOG,
                              'debug'):
            self.l2gw_ovsdb.update_connection_to_gateway(
                mock.Mock(), mock.Mock(), mock.Mock(), mock.Mock(),
                mock.ANY, mock.ANY)
            get_rand.assert_called_with(128)
            send_n_receive.assert_called_with(mock.ANY,
                                              self.op_id, mock.ANY, True)
            self.assertTrue(get_bindings.called)

    def test_get_bindings_to_update1(self):
        """Test case to test _get_bindings_to_update."""
        fake_op_method = 'CREATE'
        with mock.patch.object(
                ovsdb_writer.OVSDBWriter,
                '_form_logical_switch') as form_ls, \
                mock.patch.object(
                    ovsdb_writer.OVSDBWriter,
                    '_form_physical_locators') as form_pl, \
                mock.patch.object(
                    ovsdb_writer.OVSDBWriter, '_form_ports') as form_pp, \
                mock.patch.object(
                    ovsdb_schema, 'LogicalSwitch') as mock_ls, \
                mock.patch.object(
                    ovsdb_schema, 'PhysicalLocator') as mock_pl, \
                mock.patch.object(
                    ovsdb_schema, 'UcastMacsRemote') as mock_ucmr, \
                mock.patch.object(ovsdb_schema, 'PhysicalPort') as mock_pp:
            ls = mock_ls.return_value = ovsdb_schema.LogicalSwitch(
                'ls_uuid', 'ls_name', 'ls_key', 'ls_desc')
            pl = mock_pl.return_value = ovsdb_schema.PhysicalLocator(
                'pl_uuid', 'pl_dst_ip')
            mock_ucmr.return_value = ovsdb_schema.UcastMacsRemote(
                'ucmr_uuid', 'ucmr_mac', 'ucmr_ls_id', 'ucmr_pl_id', 'ucmr_ip')
            pp = mock_pp.return_value = ovsdb_schema.PhysicalPort(
                'pp_uuid', 'pp_name', 'pp_ps_id', 'pp_vlan_bindings')
            form_ls.return_value = ['uuid', ls.uuid]
            self.l2gw_ovsdb._get_bindings_to_update(mock.MagicMock(),
                                                    [{'uuid': 'uuid',
                                                      'dst_ip': 'dst_ip'}],
                                                    mock.MagicMock(),
                                                    [{'uuid': 'uuid',
                                                      'name': 'name',
                                                      'physical_switch_id':
                                                      'physical_switch_id',
                                                      'vlan_bindings':
                                                      'vlan_bindings',
                                                      'port_fault_status':
                                                      'port_fault_status'}],
                                                    fake_op_method)

            form_ls.assert_called_with(ls, mock.ANY)
            form_pl.assert_called_with(['uuid', ls.uuid], [pl],
                                       mock.ANY, mock.ANY)
            form_pp.assert_called_with(['uuid', ls.uuid], [pp], mock.ANY,
                                       fake_op_method)

    def test_get_physical_locator_dict(self):
        """Test case to test _get_physical_locator_dict."""
        fake_locator = mock.Mock()
        fake_locator.uuid = 'fake_uuid'
        fake_locator.dst_ip = 'fake_dst_ip'
        phy_loc_dict = self.l2gw_ovsdb._get_physical_locator_dict(
            fake_locator)
        self.assertEqual(phy_loc_dict['row']['dst_ip'], fake_locator.dst_ip)
        self.assertEqual(phy_loc_dict['uuid-name'], fake_locator.uuid)

    def test_get_logical_switch_dict(self):
        """Test case to test _get_logical_switch_dict."""
        fake_ls = mock.Mock()
        fake_ls.uuid = 'fake_uuid'
        fake_ls.description = 'fake_desc'
        fake_ls.name = 'fake_name'
        fake_ls.key = 100
        logical_switch_dict = self.l2gw_ovsdb._get_logical_switch_dict(
            fake_ls)
        self.assertEqual(logical_switch_dict['uuid-name'], fake_ls.uuid)
        self.assertEqual(logical_switch_dict['row']['description'],
                         fake_ls.description)
        self.assertEqual(logical_switch_dict['row']['name'],
                         fake_ls.name)
        self.assertEqual(logical_switch_dict['row']['tunnel_key'],
                         fake_ls.key)

    def test_get_ucast_macs_remote_dict(self):
        """Test case to test _get_ucast_macs_remote_dict."""
        fake_mac = mock.Mock()
        fake_mac.mac = 'fake_mac'
        fake_mac.ip_address = 'fake_ip'
        locator_list = ['fake_list']
        logical_switch_list = ['fake_switch_list']
        ucast_mac_remote_dict = self.l2gw_ovsdb._get_ucast_macs_remote_dict(
            fake_mac, locator_list, logical_switch_list)
        self.assertEqual(ucast_mac_remote_dict['row']['MAC'], fake_mac.mac)
        self.assertEqual(ucast_mac_remote_dict['row']['ipaddr'],
                         fake_mac.ip_address)
        self.assertEqual(ucast_mac_remote_dict['row']['locator'], locator_list)
        self.assertEqual(ucast_mac_remote_dict['row']['logical_switch'],
                         logical_switch_list)

    def test_get_dict_for_update_ucast_mac_remote(self):
        """Test case to test _get_dict_for_update_ucast_mac_remote."""
        fake_mac = mock.Mock()
        fake_mac.uuid = 'fake_uuid'
        fake_mac.ip_address = 'fake_ip'
        locator_list = ['fake_list']
        temp_method = self.l2gw_ovsdb._get_dict_for_update_ucast_mac_remote
        ucast_mac_remote_dict = temp_method(
            fake_mac, locator_list)
        self.assertEqual(ucast_mac_remote_dict['row']['locator'], locator_list)
        self.assertEqual(ucast_mac_remote_dict['where'],
                         [["_uuid", "==", ["uuid", fake_mac.uuid]]])

    def test_recv_data(self):
        """Test case to test _recv_data with a valid data."""
        fake_data = {"fake_key": "fake_value"}
        fake_socket = base_test.SocketClass(None,
                                            None,
                                            None,
                                            base_test.FakeDecodeClass(
                                                jsonutils.dumps(fake_data)))
        with mock.patch.object(socket, 'socket', return_value=fake_socket):
            ovsdb_conf = base_test.FakeConf()
            l2gw_obj = ovsdb_writer.OVSDBWriter(
                cfg.CONF.ovsdb, ovsdb_conf)
            result = l2gw_obj._recv_data(mock.ANY)
            self.assertEqual(jsonutils.dumps(fake_data), result)

    def test_recv_data_with_empty_data(self):
        """Test case to test _recv_data with empty data."""
        fake_socket = base_test.SocketClass(None,
                                            None,
                                            None,
                                            '')
        with mock.patch.object(socket, 'socket',
                               return_value=fake_socket):
            with mock.patch.object(ovsdb_writer.LOG, 'warning'):
                ovsdb_conf = base_test.FakeConf()
                l2gw_obj = ovsdb_writer.OVSDBWriter(
                    cfg.CONF.ovsdb, ovsdb_conf)
                result = l2gw_obj._recv_data(mock.ANY)
                self.assertIsNone(result)

    def test_recv_data_with_socket_error(self):
        """Test case to test _recv_data with socket error."""

        fake_socket = base_test.SocketClass(None,
                                            None,
                                            socket.error)
        with mock.patch.object(socket, 'socket', return_value=fake_socket):
            with mock.patch.object(ovsdb_writer.LOG,
                                   'warning') as fake_warn:
                ovsdb_conf = base_test.FakeConf()
                l2gw_obj = ovsdb_writer.OVSDBWriter(
                    cfg.CONF.ovsdb, ovsdb_conf)
                result = l2gw_obj._recv_data(mock.ANY)
                self.assertIsNone(result)
                fake_warn.assert_called_with(
                    "Did not receive any reply from the OVSDB server")



# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random
import socket

from oslo_log import log as logging
from oslo_serialization import jsonutils
from oslo_utils import excutils

from networking_l2gw.services.l2gateway.agent.ovsdb import base_connection
from networking_l2gw.services.l2gateway.common import constants as n_const
from networking_l2gw.services.l2gateway.common import ovsdb_schema
from networking_l2gw.services.l2gateway import exceptions

LOG = logging.getLogger(__name__)


class OVSDBWriter(base_connection.BaseConnection):
    """Performs transactions to OVSDB server tables."""
    def __init__(self, conf, gw_config, mgr=None):
        super(OVSDBWriter, self).__init__(conf, gw_config, mgr=None)
        self.mgr = mgr

    def _process_response(self, op_id):
        result = self._response(op_id)
        error = result.get("error", None)
        if error:
            raise exceptions.OVSDBError(
                message="Error from the OVSDB server: %s" % error
            )
        # Check errors in responses of all the subqueries
        outcomes = result.get("result", None)
        if outcomes:
            for outcome in outcomes:
                error = outcome.get("error", None)
                if error:
                    raise exceptions.OVSDBError(
                        message="Error from the OVSDB server: %s" % error)
        return result

    def _get_reply(self, operation_id, ovsdb_identifier):
        count = 0
        while count <= n_const.MAX_RETRIES:
            response = self._recv_data(ovsdb_identifier)
            LOG.debug("Response from OVSDB server = %s", str(response))
            if response:
                try:
                    json_m = jsonutils.loads(response)
                    self.responses.append(json_m)
                    method_type = json_m.get('method', None)
                    if method_type == "echo" and self.enable_manager:
                        self.ovsdb_dicts.get(ovsdb_identifier).send(
                            jsonutils.dumps(
                                {"result": json_m.get("params", None),
                                 "error": None, "id": json_m['id']}))
                    else:
                        if self._process_response(operation_id):
                            return True
                except Exception as ex:
                    with excutils.save_and_reraise_exception():
                        LOG.exception("Exception while receiving the "
                                      "response for the write request:"
                                      " [%s]", ex)
            count += 1
        with excutils.save_and_reraise_exception():
            LOG.error("Could not obtain response from the OVSDB server "
                      "for the request")

    def _send_and_receive(self, query, operation_id, ovsdb_identifier,
                          rcv_required):
        if not self.send(query, addr=ovsdb_identifier):
            return
        if rcv_required:
            self._get_reply(operation_id, ovsdb_identifier)

    def delete_logical_switch(self, logical_switch_uuid, ovsdb_identifier,
                              rcv_required=True):
        """Delete an entry from Logical_Switch OVSDB table."""
        commit_dict = {"op": "commit", "durable": True}
        op_id = str(random.getrandbits(128))
        query = {"method": "transact",
                 "params": [n_const.OVSDB_SCHEMA_NAME,
                            {"op": "delete",
                             "table": "Logical_Switch",
                             "where": [["_uuid", "==",
                                        ["uuid", logical_switch_uuid]]]},
                            commit_dict],
                 "id": op_id}
        LOG.debug("delete_logical_switch: query: %s", query)
        self._send_and_receive(query, op_id, ovsdb_identifier, rcv_required)

    def insert_ucast_macs_remote(self, l_switch_dict, locator_dict,
                                 mac_dict, ovsdb_identifier,
                                 rcv_required=True):
        """Insert an entry in Ucast_Macs_Remote OVSDB table."""
        # To insert an entry in Ucast_Macs_Remote table, it requires
        # corresponding entry in Physical_Locator (Compute node VTEP IP)
        # and Logical_Switch (Neutron network) tables.
        logical_switch = ovsdb_schema.LogicalSwitch(l_switch_dict['uuid'],
                                                    l_switch_dict['name'],
                                                    l_switch_dict['key'],
                                                    l_switch_dict['description'
                                                                  ])
        locator = ovsdb_schema.PhysicalLocator(locator_dict['uuid'],
                                               locator_dict['dst_ip'])
        macObject = ovsdb_schema.UcastMacsRemote(mac_dict['uuid'],
                                                 mac_dict['mac'],
                                                 mac_dict['logical_switch_id'],
                                                 mac_dict['physical_locator_id'
                                                          ],
                                                 mac_dict['ip_address'])
        # Form the insert query now.
        commit_dict = {"op": "commit", "durable": True}
        op_id = str(random.getrandbits(128))
        params = [n_const.OVSDB_SCHEMA_NAME]

        if locator.uuid:
            locator_list = ['uuid', locator.uuid]
        else:
            locator.uuid = ''.join(['a', str(random.getrandbits(128))])
            locator_list = ["named-uuid", locator.uuid]
            params.append(self._get_physical_locator_dict(locator))

        if logical_switch.uuid:
            l_switches = ['uuid', logical_switch.uuid]
        else:
            logical_switch.uuid = ''.join(['a', str(random.getrandbits(128))])
            l_switches = ["named-uuid", logical_switch.uuid]
            params.append(self._get_logical_switch_dict(logical_switch))

        params.append(self._get_ucast_macs_remote_dict(
            macObject, locator_list, l_switches))
        params.append(commit_dict)
        query = {"method": "transact",
                 "params": params,
                 "id": op_id}
        LOG.debug("insert_ucast_macs_remote: query: %s", query)
        self._send_and_receive(query, op_id, ovsdb_identifier, rcv_required)

    def update_ucast_macs_remote(self, locator_dict, mac_dict,
                                 ovsdb_identifier,
                                 rcv_required=True):
        """Update an entry in Ucast_Macs_Remote OVSDB table."""
        # It is possible that the locator may not exist already.
        locator = ovsdb_schema.PhysicalLocator(locator_dict['uuid'],
                                               locator_dict['dst_ip'])
        macObject = ovsdb_schema.UcastMacsRemote(mac_dict['uuid'],
                                                 mac_dict['mac'],
                                                 mac_dict['logical_switch_id'],
                                                 mac_dict['physical_locator_id'
                                                          ],
                                                 mac_dict['ip_address'])
        # Form the insert query now.
        commit_dict = {"op": "commit", "durable": True}
        op_id = str(random.getrandbits(128))
        params = [n_const.OVSDB_SCHEMA_NAME]

        # If the physical_locator does not exist (VM moving to a new compute
        # node), then insert a new record in Physical_Locator first.
        if locator.uuid:
            locator_list = ['uuid', locator.uuid]
        else:
            locator.uuid = ''.join(['a', str(random.getrandbits(128))])
            locator_list = ["named-uuid", locator.uuid]
            params.append(self._get_physical_locator_dict(locator))

        params.append(self._get_dict_for_update_ucast_mac_remote(
            macObject, locator_list))
        params.append(commit_dict)
        query = {"method": "transact",
                 "params": params,
                 "id": op_id}
        LOG.debug("update_ucast_macs_remote: query: %s", query)
        self._send_and_receive(query, op_id, ovsdb_identifier, rcv_required)

    def delete_ucast_macs_remote(self, logical_switch_uuid, macs,
                                 ovsdb_identifier,
                                 rcv_required=True):
        """Delete entries from Ucast_Macs_Remote OVSDB table."""
        commit_dict = {"op": "commit", "durable": True}
        op_id = str(random.getrandbits(128))
        params = [n_const.OVSDB_SCHEMA_NAME]
        for mac in macs:
            sub_query = {"op": "delete",
                         "table": "Ucast_Macs_Remote",
                         "where": [["MAC",
                                    "==",
                                    mac],
                                   ["logical_switch",
                                    "==",
                                    ["uuid",
                                     logical_switch_uuid]]]}
            params.append(sub_query)
        params.append(commit_dict)
        query = {"method": "transact",
                 "params": params,
                 "id": op_id}
        LOG.debug("delete_ucast_macs_remote: query: %s", query)
        self._send_and_receive(query, op_id, ovsdb_identifier, rcv_required)

    def update_connection_to_gateway(self, logical_switch_dict,
                                     locator_dicts, mac_dicts,
                                     port_dicts, ovsdb_identifier,
                                     op_method,
                                     rcv_required=True):
        """Updates Physical Port's VNI to VLAN binding."""
        # Form the JSON Query so as to update the physical port with the
        # vni-vlan (logical switch uuid to vlan) binding
        update_dicts = self._get_bindings_to_update(logical_switch_dict,
                                                    locator_dicts,
                                                    mac_dicts,
                                                    port_dicts,
                                                    op_method)
        op_id = str(random.getrandbits(128))
        query = {"method": "transact",
                 "params": update_dicts,
                 "id": op_id}
        LOG.debug("update_connection_to_gateway: query = %s", query)
        self._send_and_receive(query, op_id, ovsdb_identifier, rcv_required)

    def _recv_data(self, ovsdb_identifier):
        chunks = []
        lc = rc = 0
        prev_char = None
        while True:
            try:
                if self.enable_manager:
                    response = self.ovsdb_dicts.get(ovsdb_identifier).recv(
                        n_const.BUFFER_SIZE)
                else:
                    response = self.socket.recv(n_const.BUFFER_SIZE)
                if response:
                    response = response.decode('utf8')
                    for i, c in enumerate(response):
                        if c == '{' and not (prev_char and
                                             prev_char == '\\'):
                            lc += 1
                        elif c == '}' and not (prev_char and
                                               prev_char == '\\'):
                            rc += 1
                        if lc == rc and lc != 0:
                            chunks.append(response[0:i + 1])
                            message = "".join(chunks)
                            return message
                        prev_char = c
                    chunks.append(response)
                else:
                    LOG.warning("Did not receive any reply from the OVSDB "
                                "server")
                    return
            except (socket.error, socket.timeout):
                LOG.warning("Did not receive any reply from the OVSDB "
                            "server")
                return

    def _get_bindings_to_update(self, l_switch_dict, locator_dicts,
                                mac_dicts, port_dicts, op_method):
        # For connection-create, there are two cases to be handled
        # Case 1: VMs exist in a network on compute nodes.
        #         Connection request will contain locators, ports, MACs and
        #         network.
        # Case 2: VMs do not exist in a network on compute nodes.
        #         Connection request will contain only ports and network
        #
        # For connection-delete, we do not need logical_switch and locators
        # information, we just need ports.
        locator_list = []
        port_list = []
        ls_list = []
        logical_switch = None
        # Convert logical switch dict to a class object
        if l_switch_dict:
            logical_switch = ovsdb_schema.LogicalSwitch(
                l_switch_dict['uuid'],
                l_switch_dict['name'],
                l_switch_dict['key'],
                l_switch_dict['description'])

        # Convert locator dicts into class objects
        for locator in locator_dicts:
            locator_list.append(ovsdb_schema.PhysicalLocator(locator['uuid'],
                                                             locator['dst_ip'])
                                )

        # Convert MAC dicts into class objects. mac_dicts is a dictionary with
        # locator VTEP IP as the key and list of MACs as the value.
        locator_macs = {}
        for locator_ip, mac_list in mac_dicts.items():
            mac_object_list = []
            for mac_dict in mac_list:
                mac_object = ovsdb_schema.UcastMacsRemote(
                    mac_dict['uuid'],
                    mac_dict['mac'],
                    mac_dict['logical_switch_id'],
                    mac_dict['physical_locator_id'],
                    mac_dict['ip_address'])
                mac_object_list.append(mac_object)
            locator_macs[locator_ip] = mac_object_list

        # Convert port dicts into class objects
        for port in port_dicts:
            phys_port = ovsdb_schema.PhysicalPort(port['uuid'],
                                                  port['name'],
                                                  port['physical_switch_id'],
                                                  port['vlan_bindings'],
                                                  port['port_fault_status'])
            port_list.append(phys_port)

        bindings = []
        bindings.append(n_const.OVSDB_SCHEMA_NAME)

        # Form the query.
        commit_dict = {"op": "commit", "durable": True}
        params = [n_const.OVSDB_SCHEMA_NAME]

        # Use logical switch
        if logical_switch:
            ls_list = self._form_logical_switch(logical_switch, params)

        # Use physical locators
        if locator_list:
            self._form_physical_locators(ls_list, locator_list, locator_macs,
                                         params)

        # Use ports
        self._form_ports(ls_list, port_list, params, op_method)
        params.append(commit_dict)
        return params

    def _form_logical_switch(self, logical_switch, params):
        ls_list = []
        if logical_switch.uuid:
            ls_list = ['uuid', logical_switch.uuid]
        else:
            logical_switch.uuid = ''.join(['a', str(random.getrandbits(128))])
            ls_list = ["named-uuid", logical_switch.uuid]
            params.append(self._get_logical_switch_dict(logical_switch))
        return ls_list

    def _form_physical_locators(self, ls_list, locator_list, locator_macs,
                                params):
        for locator in locator_list:
            if locator.uuid:
                loc_list = ['uuid', locator.uuid]
            else:
                locator.uuid = ''.join(['a', str(random.getrandbits(128))])
                loc_list = ["named-uuid", locator.uuid]
                params.append(self._get_physical_locator_dict(locator))
            macs = locator_macs.get(locator.dst_ip, None)
            if macs:
                for mac in macs:
                    query = self._get_ucast_macs_remote_dict(mac,
                                                             loc_list,
                                                             ls_list)
                    params.append(query)

    def _form_ports(self, ls_list, port_list, params, op_method):
        for port in port_list:
            port_vlan_bindings = []
            outer_list = []
            port_vlan_bindings.append("map")
            if port.vlan_bindings:
                for vlan_binding in port.vlan_bindings:
                    if vlan_binding.logical_switch_uuid:
                        outer_list.append([vlan_binding.vlan,
                                          ['uuid',
                                           vlan_binding.logical_switch_uuid]])
                    else:
                        outer_list.append([vlan_binding.vlan,
                                          ls_list])
            port_vlan_bindings.append(outer_list)
            if op_method == 'CREATE':
                update_dict = {"op": "mutate",
                               "table": "Physical_Port",
                               "where": [["_uuid", "==",
                                          ["uuid", port.uuid]]],
                               "mutations": [["vlan_bindings",
                                              "insert",
                                              port_vlan_bindings]]}
            elif op_method == 'DELETE':
                update_dict = {"op": "mutate",
                               "table": "Physical_Port",
                               "where": [["_uuid", "==",
                                         ["uuid", port.uuid]]],
                               "mutations": [["vlan_bindings",
                                              "delete",
                                              port_vlan_bindings]]}
            params.append(update_dict)

    def _get_physical_locator_dict(self, locator):
        return {"op": "insert",
                "table": "Physical_Locator",
                "uuid-name": locator.uuid,
                "row": {"dst_ip": locator.dst_ip,
                        "encapsulation_type": "vxlan_over_ipv4"}}

    def _get_logical_switch_dict(self, logical_switch):
        return {"op": "insert",
                "uuid-name": logical_switch.uuid,
                "table": "Logical_Switch",
                "row": {"description": logical_switch.description,
                        "name": logical_switch.name,
                        "tunnel_key": int(logical_switch.key)}}

    def _get_ucast_macs_remote_dict(self, mac, locator_list,
                                    logical_switch_list):
        named_string = str(random.getrandbits(128))
        return {"op": "insert",
                "uuid-name": ''.join(['a', named_string]),
                "table": "Ucast_Macs_Remote",
                "row": {"MAC": mac.mac,
                        "ipaddr": mac.ip_address,
                        "locator": locator_list,
                        "logical_switch": logical_switch_list}}

    def _get_dict_for_update_ucast_mac_remote(self, mac, locator_list):
        return {"op": "update",
                "table": "Ucast_Macs_Remote",
                "where": [["_uuid", "==",
                           ["uuid", mac.uuid]]],
                "row": {"locator": locator_list}}



# Copyright (c) 2015 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from neutron import manager
from neutron.plugins.ml2 import managers
from neutron.tests import base

from neutron_lib import context
from neutron_lib.plugins import directory

from networking_l2gw.db.l2gateway import l2gateway_db
from networking_l2gw.db.l2gateway.ovsdb import lib
from networking_l2gw.services.l2gateway.common import constants as n_const
from networking_l2gw.services.l2gateway.common import ovsdb_schema
from networking_l2gw.services.l2gateway.common import tunnel_calls
from networking_l2gw.services.l2gateway.ovsdb import data
from networking_l2gw.services.l2gateway.service_drivers import agent_api


class TestL2GatewayOVSDBCallbacks(object):

    def setUp(self):
        super(TestL2GatewayOVSDBCallbacks, self).setUp()
        self.context = context.get_admin_context()

    def test_update_ovsdb_changes(self):
        fake_activity = 1
        fake_ovsdb_data = {n_const.OVSDB_IDENTIFIER: 'fake_id'}
        with mock.patch.object(data, 'OVSDBData') as ovs_data:
            self.l2gw_callbacks.update_ovsdb_changes(self.context,
                                                     fake_activity,
                                                     fake_ovsdb_data)
            ovsdb_return_value = ovs_data.return_value
            ovsdb_return_value.update_ovsdb_changes.assert_called_with(
                self.context, fake_activity, fake_ovsdb_data)

    def test_notify_ovsdb_states(self):
        fake_ovsdb_states = {'ovsdb1': 'connected'}
        with mock.patch.object(data, 'OVSDBData') as ovs_data:
            self.l2gw_callbacks.notify_ovsdb_states(self.context,
                                                    fake_ovsdb_states)
            ovsdb_return_value = ovs_data.return_value
            ovsdb_return_value.notify_ovsdb_states.assert_called_with(
                self.context, fake_ovsdb_states)

    def test_get_ovsdbdata_object(self):
        fake_ovsdb_id = 'fake_ovsdb_id'
        with mock.patch.object(data, 'OVSDBData') as ovs_data:
            ret_value = self.l2gw_callbacks.get_ovsdbdata_object(
                fake_ovsdb_id)
            ret_value1 = ovs_data.assert_called_with(fake_ovsdb_id)
            self.assertEqual(ret_value, ret_value1)


class TestOVSDBData(base.BaseTestCase):

    def setUp(self):
        super(TestOVSDBData, self).setUp()
        self.context = context.get_admin_context()
        self.ovsdb_identifier = 'fake_ovsdb_id'
        mock.patch.object(directory, 'get_plugin').start()
        mock.patch.object(managers, 'TypeManager').start()
        self.ovsdb_data = data.OVSDBData(self.ovsdb_identifier)

    def test_init(self):
        with mock.patch.object(data.OVSDBData,
                               '_setup_entry_table') as setup_entry_table:
            self.ovsdb_data.__init__(self.ovsdb_identifier)
            self.assertEqual('fake_ovsdb_id',
                             self.ovsdb_data.ovsdb_identifier)
            self.assertTrue(setup_entry_table.called)

    def test_update_ovsdb_changes(self):
        fake_dict = {}
        fake_activity = 1
        fake_remote_mac = {'uuid': '123456',
                           'mac': 'mac123',
                           'ovsdb_identifier': 'host1',
                           'logical_switch_id': 'ls123'}
        fake_new_logical_switches = [fake_dict]
        fake_new_physical_port = [fake_dict]
        fake_new_physical_switches = [fake_dict]
        fake_new_physical_locators = [fake_dict]
        fake_new_local_macs = [fake_dict]
        fake_new_remote_macs = [fake_remote_mac]
        fake_modified_remote_macs = [fake_dict]
        fake_modified_physical_ports = [fake_dict]
        fake_modified_local_macs = [fake_dict]
        fake_deleted_logical_switches = [fake_dict]
        fake_deleted_physical_ports = [fake_dict]
        fake_deleted_physical_switches = [fake_dict]
        fake_deleted_physical_locators = [fake_dict]
        fake_deleted_local_macs = [fake_dict]
        fake_deleted_remote_macs = [fake_dict]

        fake_ovsdb_data = {
            n_const.OVSDB_IDENTIFIER: 'fake_ovsdb_id',
            'new_logical_switches': fake_new_logical_switches,
            'new_physical_ports': fake_new_physical_port,
            'new_physical_switches': fake_new_physical_switches,
            'new_physical_locators': fake_new_physical_locators,
            'new_local_macs': fake_new_local_macs,
            'new_remote_macs': fake_new_remote_macs,
            'modified_remote_macs': fake_modified_remote_macs,
            'modified_physical_ports': fake_modified_physical_ports,
            'modified_local_macs': fake_modified_local_macs,
            'deleted_logical_switches': fake_deleted_logical_switches,
            'deleted_physical_switches': fake_deleted_physical_switches,
            'deleted_physical_ports': fake_deleted_physical_ports,
            'deleted_physical_locators': fake_deleted_physical_locators,
            'deleted_local_macs': fake_deleted_local_macs,
            'deleted_remote_macs': fake_deleted_remote_macs}
        with mock.patch.object(
                    self.ovsdb_data,
                    '_process_new_logical_switches'
        ) as process_new_logical_switches, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_new_physical_ports'
        ) as process_new_physical_ports, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_new_physical_switches'
        ) as process_new_physical_switches, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_new_physical_locators'
        ) as process_new_physical_locators, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_new_local_macs'
        ) as process_new_local_macs, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_new_remote_macs'
        ) as process_new_remote_macs, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_modified_remote_macs'
        ) as process_modified_remote_macs, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_modified_physical_ports'
        ) as process_modified_physical_ports, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_deleted_logical_switches'
        ) as process_deleted_logical_switches, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_deleted_physical_switches'
        ) as process_deleted_physical_switches, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_deleted_physical_ports'
        ) as process_deleted_physical_ports, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_deleted_physical_locators'
        ) as process_deleted_physical_locators, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_deleted_local_macs'
        ) as process_deleted_local_macs, \
            mock.patch.object(
                self.ovsdb_data,
                '_process_deleted_remote_macs'
        ) as process_deleted_remote_macs, \
                mock.patch.object(
                    self.ovsdb_data,
                    '_handle_l2pop') as mock_handle_l2pop:
            self.ovsdb_data.entry_table = {
                'new_logical_switches': process_new_logical_switches,
                'new_physical_ports': process_new_physical_ports,
                'new_physical_switches': process_new_physical_switches,
                'new_physical_locators': process_new_physical_locators,
                'new_local_macs': process_new_local_macs,
                'new_remote_macs': process_new_remote_macs,
                'modified_remote_macs': process_modified_remote_macs,
                'modified_physical_ports': process_modified_physical_ports,
                'deleted_logical_switches': process_deleted_logical_switches,
                'deleted_physical_switches': process_deleted_physical_switches,
                'deleted_physical_ports': process_deleted_physical_ports,
                'deleted_physical_locators': process_deleted_physical_locators,
                'deleted_local_macs': process_deleted_local_macs,
                'deleted_remote_macs': process_deleted_remote_macs}
            self.ovsdb_data.update_ovsdb_changes(
                self.context, fake_activity, fake_ovsdb_data)

            process_new_logical_switches.assert_called_with(
                self.context, fake_new_logical_switches)
            process_new_physical_ports.assert_called_with(
                self.context, fake_new_physical_port)
            process_new_physical_switches.assert_called_with(
                self.context, fake_new_physical_switches)
            process_new_physical_locators.assert_called_with(
                self.context, fake_new_physical_locators)
            process_new_local_macs.assert_called_with(
                self.context, fake_new_local_macs)
            process_new_remote_macs.assert_called_with(
                self.context, fake_new_remote_macs)
            process_modified_remote_macs.assert_called_with(
                self.context, fake_modified_remote_macs)
            process_modified_physical_ports.assert_called_with(
                self.context, fake_modified_physical_ports)
            process_deleted_logical_switches.assert_called_with(
                self.context, fake_deleted_logical_switches)
            process_deleted_physical_switches.assert_called_with(
                self.context, fake_deleted_physical_switches)
            process_deleted_physical_ports.assert_called_with(
                self.context, fake_deleted_physical_ports)
            process_deleted_physical_locators.assert_called_with(
                self.context, fake_deleted_physical_locators)
            process_deleted_local_macs.assert_called_with(
                self.context, fake_deleted_local_macs)
            process_deleted_remote_macs.assert_called_with(
                self.context, fake_deleted_remote_macs)
            self.assertTrue(mock_handle_l2pop.called)

    @mock.patch.object(lib, 'get_all_pending_remote_macs_in_asc_order')
    @mock.patch.object(lib, 'delete_pending_ucast_mac_remote')
    @mock.patch.object(ovsdb_schema, 'LogicalSwitch')
    @mock.patch.object(ovsdb_schema, 'PhysicalLocator')
    @mock.patch.object(ovsdb_schema, 'UcastMacsRemote')
    @mock.patch.object(agent_api.L2gatewayAgentApi, 'add_vif_to_gateway')
    @mock.patch.object(agent_api.L2gatewayAgentApi, 'update_vif_to_gateway')
    @mock.patch.object(agent_api.L2gatewayAgentApi, 'delete_vif_from_gateway')
    def test_notify_ovsdb_states(self, mock_del_vif, mock_upd_vif,
                                 mock_add_vif, mock_ucmr, mock_pl,
                                 mock_ls, mock_del_pend_recs,
                                 mock_get_pend_recs):
        fake_ovsdb_states = {'ovsdb1': 'connected'}
        fake_dict = {'logical_switch_uuid': 'fake_ls_id',
                     'mac': 'fake_mac',
                     'locator_uuid': 'fake_loc_id',
                     'dst_ip': 'fake_dst_ip',
                     'vm_ip': 'fake_vm_ip'}
        fake_insert_dict = {'operation': 'insert'}
        fake_insert_dict.update(fake_dict)
        fake_update_dict = {'operation': 'update'}
        fake_update_dict.update(fake_dict)
        fake_delete_dict = {'operation': 'delete'}
        fake_delete_dict.update(fake_dict)
        mock_get_pend_recs.return_value = [fake_insert_dict]
        self.ovsdb_data.notify_ovsdb_states(
            self.context, fake_ovsdb_states)
        self.assertTrue(mock_add_vif.called)
        mock_get_pend_recs.return_value = [fake_update_dict]
        self.ovsdb_data.notify_ovsdb_states(
            self.context, fake_ovsdb_states)
        self.assertTrue(mock_upd_vif.called)
        mock_get_pend_recs.return_value = [fake_delete_dict]
        self.ovsdb_data.notify_ovsdb_states(
            self.context, fake_ovsdb_states)
        self.assertTrue(mock_del_vif.called)

    def test_process_new_logical_switches(self):
        fake_dict = {}
        fake_new_logical_switches = [fake_dict]
        with mock.patch.object(lib, 'get_logical_switch',
                               return_value=None) as get_ls:
            with mock.patch.object(lib,
                                   'add_logical_switch') as add_ls:
                self.ovsdb_data._process_new_logical_switches(
                    self.context, fake_new_logical_switches)
                self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
                self.assertEqual('fake_ovsdb_id',
                                 fake_dict[n_const.OVSDB_IDENTIFIER])
                get_ls.assert_called_with(self.context, fake_dict)
                add_ls.assert_called_with(self.context, fake_dict)

    def test_process_new_physical_switches(self):
        fake_dict = {'tunnel_ip': ['set']}
        fake_new_physical_switches = [fake_dict]
        with mock.patch.object(lib, 'get_physical_switch',
                               return_value=None) as get_ps:
            with mock.patch.object(lib,
                                   'add_physical_switch') as add_ps:
                self.ovsdb_data._process_new_physical_switches(
                    self.context, fake_new_physical_switches)
                self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
                self.assertIsNone(fake_dict['tunnel_ip'])
                self.assertEqual('fake_ovsdb_id',
                                 fake_dict[n_const.OVSDB_IDENTIFIER])
                get_ps.assert_called_with(self.context, fake_dict)
                add_ps.assert_called_with(self.context, fake_dict)

    @mock.patch.object(lib, 'get_physical_port', return_value=None)
    @mock.patch.object(lib, 'add_physical_port')
    @mock.patch.object(lib, 'get_vlan_binding', return_value=None)
    @mock.patch.object(lib, 'add_vlan_binding')
    def test_process_new_physical_ports(self, add_vlan, get_vlan,
                                        add_pp, get_pp):
        fake_dict1 = {}
        fake_dict2 = {'vlan_bindings': [fake_dict1]}
        fake_new_physical_ports = [fake_dict2]
        self.ovsdb_data._process_new_physical_ports(
            self.context, fake_new_physical_ports)
        self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict2)
        self.assertEqual('fake_ovsdb_id',
                         fake_dict2[n_const.OVSDB_IDENTIFIER])
        get_pp.assert_called_with(self.context, fake_dict2)
        add_pp.assert_called_with(self.context, fake_dict2)
        self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict1)
        self.assertIn('port_uuid', fake_dict1)
        get_vlan.assert_called_with(self.context, fake_dict1)
        add_vlan.assert_called_with(self.context, fake_dict1)

    def test_process_new_physical_locators(self):
        fake_dict = {}
        fake_new_physical_locators = [fake_dict]
        with mock.patch.object(lib, 'get_physical_locator',
                               return_value=None) as get_pl:
            with mock.patch.object(lib,
                                   'add_physical_locator') as add_pl:
                self.ovsdb_data._process_new_physical_locators(
                    self.context, fake_new_physical_locators)
                self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
                self.assertEqual('fake_ovsdb_id',
                                 fake_dict[n_const.OVSDB_IDENTIFIER])
                get_pl.assert_called_with(self.context, fake_dict)
                add_pl.assert_called_with(self.context, fake_dict)

    @mock.patch.object(lib, 'get_ucast_mac_local', return_value=None)
    @mock.patch.object(lib, 'add_ucast_mac_local')
    def test_process_new_local_macs(self, add_lm, get_lm):
        fake_dict = {'uuid': '123456',
                     'mac': 'mac123',
                     'ovsdb_identifier': 'host1',
                     'logical_switch_id': 'ls123'}
        fake_new_local_macs = [fake_dict]
        self.ovsdb_data._process_new_local_macs(
            self.context, fake_new_local_macs)
        self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
        self.assertEqual('fake_ovsdb_id',
                         fake_dict[n_const.OVSDB_IDENTIFIER])
        get_lm.assert_called_with(self.context, fake_dict)
        add_lm.assert_called_with(self.context, fake_dict)

    def test_process_new_remote_macs(self):
        fake_dict = {'logical_switch_id': 'ls123'}
        fake_new_remote_macs = [fake_dict]
        with mock.patch.object(lib, 'get_ucast_mac_remote',
                               return_value=None) as get_mr:
            with mock.patch.object(lib,
                                   'add_ucast_mac_remote') as add_mr:
                self.ovsdb_data._process_new_remote_macs(
                    self.context, fake_new_remote_macs)
                self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
                self.assertEqual('fake_ovsdb_id',
                                 fake_dict[n_const.OVSDB_IDENTIFIER])
                get_mr.assert_called_with(self.context, fake_dict)
                add_mr.assert_called_with(self.context, fake_dict)

    def test_process_modified_remote_macs(self):
        fake_dict = {'logical_switch_id': 'ls123'}
        fake_modified_remote_macs = [fake_dict]
        with mock.patch.object(lib,
                               'update_ucast_mac_remote') as update_mr:
            self.ovsdb_data._process_modified_remote_macs(
                self.context, fake_modified_remote_macs)
            self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
            self.assertEqual('fake_ovsdb_id',
                             fake_dict[n_const.OVSDB_IDENTIFIER])
            update_mr.assert_called_with(self.context, fake_dict)

    def test_process_deleted_logical_switches(self):
        fake_dict = {}
        fake_deleted_logical_switches = [fake_dict]
        with mock.patch.object(lib, 'delete_logical_switch') as delete_ls:
            self.ovsdb_data._process_deleted_logical_switches(
                self.context, fake_deleted_logical_switches)
            self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
            self.assertEqual('fake_ovsdb_id',
                             fake_dict[n_const.OVSDB_IDENTIFIER])
            delete_ls.assert_called_with(self.context, fake_dict)

    def test_get_agent_by_mac(self):
        fake_mac = {'mac': 'fake_mac_1'}
        fake_port = [{'binding:host_id': 'fake_host'}]
        with mock.patch.object(self.ovsdb_data, '_get_port_by_mac',
                               return_value=fake_port) as mock_get_port_mac, \
            mock.patch.object(
                self.ovsdb_data,
                '_get_agent_details_by_host') as mock_get_agent_detail:
            self.ovsdb_data._get_agent_by_mac(self.context, fake_mac)
            mock_get_port_mac.assert_called_with(self.context, 'fake_mac_1')
            mock_get_agent_detail.assert_called_with(self.context, 'fake_host')

    def test_get_agent_details_by_host(self):
        fake_agent = {'configurations': {'tunnel_types': ["vxlan"],
                      'l2_population': True}}
        fake_agents = [fake_agent]
        with mock.patch.object(self.ovsdb_data.core_plugin,
                               'get_agents',
                               return_value=fake_agents):
            l2pop_enabled = self.ovsdb_data._get_agent_details_by_host(
                self.context, 'fake_host')
            self.assertTrue(l2pop_enabled)

    def test_process_deleted_physical_switches(self):
        fake_dict = {}
        fake_deleted_physical_switches = [fake_dict]
        fake_ls_dict = {'uuid': 'ls-uuid'}
        fake_ls_list = [fake_ls_dict]
        with mock.patch.object(lib, 'delete_physical_switch') as delete_ps, \
            mock.patch.object(lib, 'get_all_physical_switches_by_ovsdb_id',
                              return_value=False) as get_ps, \
            mock.patch.object(lib, 'get_all_logical_switches_by_ovsdb_id',
                              return_value=fake_ls_list) as get_ls, \
            mock.patch.object(agent_api.L2gatewayAgentApi,
                              'delete_network') as del_network:
            self.ovsdb_data._process_deleted_physical_switches(
                self.context, fake_deleted_physical_switches)
            self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
            self.assertEqual('fake_ovsdb_id',
                             fake_dict[n_const.OVSDB_IDENTIFIER])
            delete_ps.assert_called_with(self.context, fake_dict)
            get_ps.assert_called_with(self.context, 'fake_ovsdb_id')
            get_ls.assert_called_with(self.context, 'fake_ovsdb_id')
            del_network.assert_called_with(self.context, 'fake_ovsdb_id',
                                           'ls-uuid')

    def test_process_deleted_physical_ports(self):
        fake_dict = {'name': 'fake_uuid', 'uuid': 'fake_name'}
        fake_deleted_physical_ports = [fake_dict]
        fake_physical_port = {'uuid': 'fake_uuid',
                              'name': 'fake_name'}
        fake_physical_switch = {'uuid': 'fake_uuid',
                                'ovsdb_identifier': 'fake_ovsdb_id',
                                'name': 'fake_switch'},
        fake_vlan_binding = {'port_uuid:': 'fake_port_uuid',
                             'vlan': 'fake_vlan',
                             'logical_switch_uuid': 'fake_switch_uuid',
                             'ovsdb_identifier': 'fake_ovsdb_id'}
        with mock.patch.object(lib,
                               'delete_physical_port'), \
            mock.patch.object(lib,
                              'get_physical_port',
                              return_value=fake_physical_port), \
            mock.patch.object(lib, 'get_physical_switch',
                              return_vaue=fake_physical_switch), \
            mock.patch.object(lib,
                              'get_all_vlan_bindings_by_physical_port',
                              return_vaue=fake_vlan_binding), \
            mock.patch.object(l2gateway_db.L2GatewayMixin,
                              '_get_l2gw_ids_by_interface_switch',
                              return_value=['fake_uuid']), \
            mock.patch.object(
                l2gateway_db.L2GatewayMixin,
                '_delete_connection_by_l2gw_id') as l2gw_conn_del:
            self.ovsdb_data._process_deleted_physical_ports(
                self.context, fake_deleted_physical_ports)
            self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
            self.assertEqual('fake_ovsdb_id',
                             fake_dict[n_const.OVSDB_IDENTIFIER])
            l2gw_conn_del.assert_called_with(self.context, 'fake_uuid')

    @mock.patch.object(lib, 'delete_physical_port')
    @mock.patch.object(lib, 'get_physical_port')
    @mock.patch.object(lib, 'get_physical_switch')
    @mock.patch.object(l2gateway_db.L2GatewayMixin,
                       '_get_l2gw_ids_by_interface_switch',
                       return_value=['fake_uuid'])
    @mock.patch.object(l2gateway_db.L2GatewayMixin,
                       '_delete_connection_by_l2gw_id')
    @mock.patch.object(lib,
                       'get_all_vlan_bindings_by_physical_port')
    @mock.patch.object(lib,
                       'get_all_vlan_bindings_by_logical_switch')
    @mock.patch.object(data.OVSDBData, '_delete_macs_from_ovsdb')
    @mock.patch.object(lib, 'delete_vlan_binding')
    def test_process_deleted_physical_ports_with_delete_macs(
            self, del_vlan, del_macs, get_vlan_by_ls, get_vlan_by_pp,
            l2gw_conn_del, get_l2gw, get_ps, get_pp, delete_pp):
        fake_dict = {'uuid': 'fake_uuid', 'name': 'fake_name',
                     'logical_switch_id': 'fake_ls_id',
                     'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_deleted_physical_ports = [fake_dict]
        fake_physical_port = {'uuid': 'fake_uuid',
                              'name': 'fake_name',
                              'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_physical_switch = {'uuid': 'fake_uuid',
                                'ovsdb_identifier': 'fake_ovsdb_id',
                                'name': 'fake_switch'}
        vlan_binding_dict = {'logical_switch_uuid': 'fake_ls_id',
                             'ovsdb_identifier': 'fake_ovsdb_id',
                             'port_uuid': 'fake_uuid',
                             'vlan': 'fake_vlan',
                             'logical_switch_id': 'fake_ls_id'}
        fake_vlan_binding_list = [vlan_binding_dict]
        fake_binding_list = [vlan_binding_dict]
        get_pp.return_value = fake_physical_port
        get_ps.return_vaue = fake_physical_switch
        get_vlan_by_pp.return_value = fake_vlan_binding_list
        get_vlan_by_ls.return_value = fake_binding_list
        self.ovsdb_data._process_deleted_physical_ports(
            self.context, fake_deleted_physical_ports)
        self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
        self.assertEqual('fake_ovsdb_id',
                         fake_dict[n_const.OVSDB_IDENTIFIER])
        l2gw_conn_del.assert_called_with(self.context, 'fake_uuid')
        get_vlan_by_pp.assert_called_with(self.context, fake_dict)
        del_vlan.assert_called_with(self.context, vlan_binding_dict)
        get_vlan_by_ls.assert_called_with(self.context, vlan_binding_dict)
        del_macs.assert_called_with(self.context,
                                    'fake_ls_id', 'fake_ovsdb_id')
        del_vlan.assert_called_with(self.context, vlan_binding_dict)
        delete_pp.assert_called_with(self.context, fake_dict)

    @mock.patch.object(data.OVSDBData,
                       '_get_logical_switch_ids',
                       return_value=['1'])
    @mock.patch.object(lib,
                       'get_all_physical_switches_by_ovsdb_id',
                       return_value=[{'tunnel_ip': '3.3.3.3'}])
    @mock.patch.object(data.OVSDBData,
                       '_get_fdb_entries')
    @mock.patch.object(lib,
                       'delete_physical_locator')
    @mock.patch.object(data.OVSDBData, '_get_agent_ips',
                       return_value={'1.1.1.1': 'hostname'})
    @mock.patch.object(tunnel_calls.Tunnel_Calls,
                       'trigger_l2pop_delete')
    def test_process_deleted_physical_locators(
            self, trig_l2pop, get_agent_ips, delete_pl, get_fdb, get_all_ps,
            get_ls):
        """Test case to test _process_deleted_physical_locators.

        for unicast rpc to the L2 agent
        """
        fake_dict1 = {'dst_ip': '1.1.1.1'}
        fake_dict2 = {'dst_ip': '2.2.2.2'}
        fake_deleted_physical_locators = [fake_dict2, fake_dict1]
        mock.patch.object(manager, 'NeutronManager').start()
        self.ovsdb_data._process_deleted_physical_locators(
            self.context, fake_deleted_physical_locators)
        self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict1)
        self.assertTrue(get_ls.called)
        self.assertTrue(get_all_ps.called)
        self.assertTrue(get_fdb.called)
        self.assertEqual('fake_ovsdb_id',
                         fake_dict1[n_const.OVSDB_IDENTIFIER])
        delete_pl.assert_called_with(self.context, fake_dict1)
        self.assertTrue(get_agent_ips.called)
        trig_l2pop.assert_called_with(self.context,
                                      mock.ANY,
                                      'hostname')

    @mock.patch.object(data.OVSDBData,
                       '_get_logical_switch_ids',
                       return_value=['1'])
    @mock.patch.object(lib,
                       'get_all_physical_switches_by_ovsdb_id',
                       return_value=[{'tunnel_ip': '3.3.3.3'}])
    @mock.patch.object(data.OVSDBData,
                       '_get_fdb_entries')
    @mock.patch.object(lib,
                       'delete_physical_locator')
    @mock.patch.object(data.OVSDBData, '_get_agent_ips',
                       return_value={'2.2.2.2': 'hostname'})
    @mock.patch.object(tunnel_calls.Tunnel_Calls,
                       'trigger_l2pop_delete')
    def test_process_deleted_physical_locators1(
            self, trig_l2pop, get_agent_ips, delete_pl, get_fdb,
            get_all_ps, get_ls):
        """Test case to test _process_deleted_physical_locators.

        for broadcast rpc to the L2 agents
        """
        fake_dict1 = {'dst_ip': '1.1.1.1'}
        fake_deleted_physical_locators = [fake_dict1]
        mock.patch.object(manager, 'NeutronManager').start()
        self.ovsdb_data._process_deleted_physical_locators(
            self.context, fake_deleted_physical_locators)
        self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict1)
        self.assertTrue(get_ls.called)
        self.assertTrue(get_all_ps.called)
        self.assertTrue(get_fdb.called)
        self.assertEqual('fake_ovsdb_id',
                         fake_dict1[n_const.OVSDB_IDENTIFIER])
        delete_pl.assert_called_once_with(self.context, fake_dict1)
        self.assertTrue(get_agent_ips.called)
        trig_l2pop.assert_called_with(self.context,
                                      mock.ANY)

    def test_process_deleted_local_macs(self):
        fake_dict = {'uuid': '123456',
                     'mac': 'mac123',
                     'ovsdb_identifier': 'host1',
                     'logical_switch_id': 'ls123'}
        fake_deleted_local_macs = [fake_dict]
        with mock.patch.object(lib, 'delete_ucast_mac_local') as delete_ml:
            with mock.patch.object(lib,
                                   'get_ucast_mac_remote_by_mac_and_ls',
                                   return_value=True):
                self.ovsdb_data._process_deleted_local_macs(
                    self.context, fake_deleted_local_macs)
                self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
                self.assertEqual('fake_ovsdb_id',
                                 fake_dict[n_const.OVSDB_IDENTIFIER])
                delete_ml.assert_called_with(self.context, fake_dict)

    def test_process_deleted_remote_macs(self):
        fake_dict = {}
        fake_deleted_remote_macs = [fake_dict]
        with mock.patch.object(lib, 'delete_ucast_mac_remote') as delete_mr:
            self.ovsdb_data._process_deleted_remote_macs(
                self.context, fake_deleted_remote_macs)
            self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict)
            self.assertEqual('fake_ovsdb_id',
                             fake_dict[n_const.OVSDB_IDENTIFIER])
            delete_mr.assert_called_with(self.context, fake_dict)

    @mock.patch.object(lib, 'get_physical_port')
    @mock.patch.object(lib, 'add_physical_port')
    @mock.patch.object(lib, 'get_all_vlan_bindings_by_physical_port')
    @mock.patch.object(lib, 'add_vlan_binding')
    @mock.patch.object(lib, 'update_physical_ports_status')
    def test_process_modified_physical_ports(self, update_pp_status, add_vlan,
                                             get_vlan, add_pp, get_pp):
        fake_dict1 = {}
        fake_dict2 = {'vlan_bindings': [fake_dict1],
                      'uuid': 'fake_uuid'}
        fake_modified_physical_ports = [fake_dict2]
        self.ovsdb_data._process_modified_physical_ports(
            self.context, fake_modified_physical_ports)
        self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict2)
        self.assertEqual('fake_ovsdb_id',
                         fake_dict2[n_const.OVSDB_IDENTIFIER])
        get_pp.assert_called_with(self.context, fake_dict2)
        update_pp_status.assert_called_with(self.context, fake_dict2)
        self.assertFalse(add_pp.called)
        get_vlan.assert_called_with(self.context, fake_dict2)
        self.assertIn(n_const.OVSDB_IDENTIFIER, fake_dict1)
        self.assertIn('port_uuid', fake_dict1)
        add_vlan.assert_called_with(self.context, fake_dict1)



# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron.agent import rpc as agent_rpc
from neutron_lib import context

from oslo_config import cfg
from oslo_log import log as logging
from oslo_service import loopingcall
from oslo_service import periodic_task

from networking_l2gw.services.l2gateway.agent import agent_api
from networking_l2gw.services.l2gateway.common import constants as n_const
from networking_l2gw.services.l2gateway.common import topics

LOG = logging.getLogger(__name__)


class BaseAgentManager(periodic_task.PeriodicTasks):
    """Basic agent manager that handles basic RPCs and report states."""

    def __init__(self, conf=None):
        conf = getattr(self, "conf", cfg.CONF)
        super(BaseAgentManager, self).__init__(conf)
        self.l2gw_agent_type = ''
        self.gateways = {}
        self.plugin_rpc = agent_api.L2GatewayAgentApi(
            topics.L2GATEWAY_PLUGIN,
            self.conf.host
        )
        self._get_agent_state()
        self.admin_state_up = True
        self._setup_state_rpc()

    def _get_agent_state(self):
        self.agent_state = {
            'binary': 'neutron-l2gateway-agent',
            'host': self.conf.host,
            'topic': topics.L2GATEWAY_AGENT,
            'configurations': {
                'report_interval': self.conf.AGENT.report_interval,
                n_const.L2GW_AGENT_TYPE: self.l2gw_agent_type,
            },
            'start_flag': True,
            'agent_type': n_const.AGENT_TYPE_L2GATEWAY}

    def _setup_state_rpc(self):
        self.state_rpc = agent_rpc.PluginReportStateAPI(
            topics.L2GATEWAY_PLUGIN)
        report_interval = self.conf.AGENT.report_interval
        if report_interval:
            heartbeat = loopingcall.FixedIntervalLoopingCall(
                self._report_state)
            heartbeat.start(interval=report_interval)

    def _report_state(self):
        try:
            ctx = context.get_admin_context_without_session()
            self.state_rpc.report_state(ctx, self.agent_state,
                                        True)
            self.agent_state['start_flag'] = False
        except Exception:
            LOG.exception("Failed reporting state!")
            self.handle_report_state_failure()

    def handle_report_state_failure(self):
        pass

    def agent_updated(self, context, payload):
        LOG.info("agent_updated by server side %s!", payload)

    def set_monitor_agent(self, context, hostname):
        """Handle RPC call from plugin to update agent type.

        RPC call from the plugin to accept that I am a monitoring
        or a transact agent. This is a fanout cast message
        """
        if hostname == self.conf.host:
            self.l2gw_agent_type = n_const.MONITOR
        else:
            self.l2gw_agent_type = ''

        self.agent_state.get('configurations')[n_const.L2GW_AGENT_TYPE
                                               ] = self.l2gw_agent_type



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""Initial no-op Liberty contract rule.

Revision ID: 79919185aa99
Revises: kilo
Create Date: 2015-07-16 00:00:00.000000

"""

from neutron.db import migration
from neutron.db.migration import cli


# revision identifiers, used by Alembic.
revision = '79919185aa99'
down_revision = 'kilo'
branch_labels = (cli.CONTRACT_BRANCH,)

# milestone identifier, used by neutron-db-manage
neutron_milestone = [migration.LIBERTY, migration.MITAKA]


def upgrade():
    pass



# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class PhysicalLocator(object):
    def __init__(self, uuid, dst_ip):
        self.uuid = uuid
        self.dst_ip = dst_ip


class PhysicalSwitch(object):
    def __init__(self, uuid, name, tunnel_ip, switch_fault_status):
        self.uuid = uuid
        self.name = name
        self.tunnel_ip = tunnel_ip
        self.switch_fault_status = switch_fault_status


class PhysicalPort(object):
    def __init__(self, uuid, name, phys_switch_id, vlan_binding_dicts,
                 port_fault_status):
        self.uuid = uuid
        self.name = name
        self.physical_switch_id = phys_switch_id
        self.vlan_bindings = []
        self.port_fault_status = port_fault_status
        if vlan_binding_dicts:
            for vlan_binding in vlan_binding_dicts:
                v_binding = VlanBinding(vlan_binding['vlan'],
                                        vlan_binding['logical_switch_uuid'])
                self.vlan_bindings.append(v_binding)


class LogicalSwitch(object):
    def __init__(self, uuid, name, key, description):
        self.uuid = uuid
        self.name = name
        self.key = key
        self.description = description


class UcastMacsLocal(object):
    def __init__(self, uuid, mac, logical_switch_id, physical_locator_id,
                 ip_address):
        self.uuid = uuid
        self.mac = mac
        self.logical_switch_id = logical_switch_id
        self.physical_locator_id = physical_locator_id
        self.ip_address = ip_address


class UcastMacsRemote(object):
    def __init__(self, uuid, mac, logical_switch_id, physical_locator_id,
                 ip_address):
        self.uuid = uuid
        self.mac = mac
        self.logical_switch_id = logical_switch_id
        self.physical_locator_id = physical_locator_id
        self.ip_address = ip_address


class VlanBinding(object):
    def __init__(self, vlan, logical_switch_uuid):
        self.vlan = vlan
        self.logical_switch_uuid = logical_switch_uuid


class McastMacsLocal(object):
    def __init__(self, uuid, mac, logical_switch, locator_set,
                 ip_address):
        self.uuid = uuid
        self.mac = mac
        self.logical_switch_id = logical_switch
        self.locator_set = locator_set
        self.ip_address = ip_address


class PhysicalLocatorSet(object):
    def __init__(self, uuid, locators):
        self.uuid = uuid
        self.locators = locators



# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os.path
import socket
import ssl
import time
from unittest import mock

import eventlet
from neutron.tests import base
from oslo_config import cfg
from oslo_log import log as logging
from oslo_serialization import jsonutils

from networking_l2gw.services.l2gateway.agent import l2gateway_config as conf
from networking_l2gw.services.l2gateway.agent.ovsdb import base_connection
from networking_l2gw.services.l2gateway.agent.ovsdb import manager
from networking_l2gw.services.l2gateway.common import config
from networking_l2gw.services.l2gateway.common import constants as n_const

LOG = logging.getLogger(__name__)


class FakeConf(object):
    def __init__(self):
        self.use_ssl = False
        self.ovsdb_ip = '1.1.1.1'
        self.ovsdb_port = 6632


class FakeDecodeClass(object):
    def __init__(self, fake_data):
        self.fake_data = fake_data

    def decode(self, utf8):
        return self.fake_data


class SocketClass(object):
    def __init__(self,
                 connect_error=None,
                 send_error=None,
                 recv_error=None,
                 rcv_data=None):
        self.connect_error = connect_error
        self.rcv_data = rcv_data
        self.send_error = send_error
        self.recv_error = recv_error

    def connect(self, ip_port):
        if self.connect_error:
            raise self.connect_error

    def send(self, data):
        if self.send_error:
            raise self.send_error
        return len(data)

    def recv(self, length):
        if self.recv_error:
            raise self.recv_error
        return self.rcv_data

    def close(self):
        pass


class TestBaseConnection(base.BaseTestCase):
    def setUp(self):
        super(TestBaseConnection, self).setUp()

        self.conf = mock.patch.object(conf, 'L2GatewayConfig').start()
        config.register_ovsdb_opts_helper(cfg.CONF)
        cfg.CONF.set_override('max_connection_retries', 0, 'ovsdb')

        self.sock = mock.patch('socket.socket').start()
        self.ssl_sock = mock.patch.object(ssl, 'wrap_socket').start()
        self.l2gw_ovsdb = base_connection.BaseConnection(mock.Mock(),
                                                         self.conf)
        self.op_id = 'abcd'
        self.fake_message = {'id': self.op_id,
                             'fake_key': 'fake_value'}

        self.l2gw_ovsdb.responses = [self.fake_message]

    def test_init(self):
        """Test case to test __init__."""

        fakesocket = SocketClass()
        with mock.patch.object(base_connection.LOG, 'debug') as logger_call, \
                mock.patch.object(socket, 'socket',
                                  return_value=fakesocket):
            self.l2gw_ovsdb.__init__(mock.Mock(), self.conf)
            self.assertTrue(self.l2gw_ovsdb.connected)
            self.assertTrue(logger_call.called)
            self.assertTrue(self.sock.called)

    def test_init_with_socket_error(self):
        """Test case to test __init__ with socket error exception."""

        fakesocket = SocketClass(socket.error)
        with mock.patch.object(base_connection.LOG,
                               'exception') as logger_exc, \
                mock.patch.object(base_connection.LOG,
                                  'warning') as logger_warn, \
                mock.patch.object(socket, 'socket',
                                  return_value=fakesocket) as sock_connect, \
                mock.patch.object(time, 'sleep'):
            ovsdb_conf = FakeConf()
            self.assertRaises(socket.error, base_connection.BaseConnection,
                              cfg.CONF.ovsdb, ovsdb_conf)
            self.assertTrue(logger_warn.called)
            self.assertTrue(logger_exc.called)
            self.assertTrue(sock_connect.called)

    def test_init_with_timeout(self):
        """Test case to test __init__ with socket timeout exception."""

        fakesocket = SocketClass(socket.timeout)
        with mock.patch.object(base_connection.LOG,
                               'exception') as logger_exc, \
                mock.patch.object(base_connection.LOG,
                                  'warning') as logger_warn, \
                mock.patch.object(socket, 'socket',
                                  return_value=fakesocket) as sock_connect, \
                mock.patch.object(time, 'sleep'):
            ovsdb_conf = FakeConf()
            self.assertRaises(socket.timeout, base_connection.BaseConnection,
                              cfg.CONF.ovsdb, ovsdb_conf)
            self.assertTrue(logger_warn.called)
            self.assertTrue(logger_exc.called)
            self.assertTrue(sock_connect.called)

    def test_response(self):
        """Test case to test _response."""
        response = self.l2gw_ovsdb._response(self.op_id)
        self.assertIsNotNone(response)
        self.assertEqual(response, self.fake_message)

    def test_send(self):
        """Test case to test send."""
        with mock.patch.object(self.l2gw_ovsdb.socket, 'send',
                               side_effect=Exception) as send:
            with mock.patch.object(base_connection.LOG,
                                   'exception'):
                with mock.patch.object(self.l2gw_ovsdb,
                                       'disconnect') as mock_disconnect:
                    with mock.patch.object(base_connection.LOG,
                                           'warning'):
                        self.l2gw_ovsdb.send(mock.Mock())
                        self.assertTrue(send.called)
                        self.assertTrue(mock_disconnect.called)

    def test_disconnect(self):
        """Test case to test disconnect socket."""
        self.l2gw_ovsdb.monitor = True
        with mock.patch.object(self.l2gw_ovsdb.socket, 'close') as sock_close:
            self.l2gw_ovsdb.disconnect()
            self.assertTrue(sock_close.called)
            self.assertFalse(self.l2gw_ovsdb.connected)


class TestBaseConnection_with_enable_manager(base.BaseTestCase):
    def setUp(self):
        super(TestBaseConnection_with_enable_manager, self).setUp()

        self.conf = mock.patch.object(conf, 'L2GatewayConfig').start()
        config.register_ovsdb_opts_helper(cfg.CONF)
        cfg.CONF.set_override('enable_manager', True, 'ovsdb')
        self.mgr = mock.patch.object(manager, 'OVSDBManager').start()
        self.l2gw_ovsdb_conn = base_connection.BaseConnection(
            mock.Mock(), self.conf, self.mgr)
        self.mock_sock = mock.patch('socket.socket').start()
        self.l2gw_ovsdb_conn.s = mock.patch('socket.socket').start()
        self.fakesocket = SocketClass()
        self.fake_ip = 'fake_ip'
        self.l2gw_ovsdb_conn.ovsdb_conn_list = ['fake_ip']
        self.l2gw_ovsdb_conn.ovsdb_dicts = {'fake_ip': self.fakesocket}

    def test_init_with_enable_manager(self):
        fake_dict = {}
        with mock.patch.object(eventlet.greenthread,
                               'spawn') as (mock_thread):
            self.l2gw_ovsdb_conn.__init__(mock.Mock(), self.conf)
            self.assertEqual(mock.ANY, self.l2gw_ovsdb_conn.s)
            self.assertFalse(self.l2gw_ovsdb_conn.check_sock_rcv)
            self.assertIsNone(self.l2gw_ovsdb_conn.check_c_sock)
            self.assertEqual(fake_dict, self.l2gw_ovsdb_conn.ovsdb_dicts)
            self.assertEqual(fake_dict, self.l2gw_ovsdb_conn.ovsdb_fd_states)
            self.assertEqual(6632,
                             self.l2gw_ovsdb_conn.manager_table_listening_port)
            self.assertTrue(mock_thread.called)

    def test_send_monitor_msg_to_ovsdb_connection(self):
        fake_ip = 'fake_ip'
        self.l2gw_ovsdb_conn.ovsdb_fd_states = {fake_ip: 'fake_status'}
        self.l2gw_ovsdb_conn.mgr.l2gw_agent_type = n_const.MONITOR
        self.l2gw_ovsdb_conn.ovsdb_conn_list = [fake_ip]
        with mock.patch.object(eventlet.greenthread, 'spawn_n') as (
                mock_thread):
            self.l2gw_ovsdb_conn._send_monitor_msg_to_ovsdb_connection(
                fake_ip)
            self.assertTrue(mock_thread.called)

    def test_exception_for_send_monitor_msg_to_ovsdb_connection(self):
        fake_ip = 'fake_ip'
        self.l2gw_ovsdb_conn.ovsdb_fd_states = {fake_ip: 'fake_status'}
        self.l2gw_ovsdb_conn.mgr.l2gw_agent_type = n_const.MONITOR
        self.l2gw_ovsdb_conn.ovsdb_conn_list = [fake_ip]
        with mock.patch.object(eventlet.greenthread, 'spawn_n',
                               side_effect=Exception) as mock_thread, \
                mock.patch.object(base_connection.LOG,
                                  'warning') as mock_warning, \
                mock.patch.object(self.l2gw_ovsdb_conn,
                                  'disconnect') as mock_disconnect:
            self.l2gw_ovsdb_conn._send_monitor_msg_to_ovsdb_connection(
                fake_ip)
            self.assertTrue(mock_thread.called)
            self.assertTrue(mock_warning.called)
            mock_disconnect.assert_called_with(fake_ip)

    def test_echo_response(self):
        fake_resp = {"method": "echo",
                     "params": "fake_params",
                     "id": "fake_id",
                     }
        with mock.patch.object(eventlet.greenthread, 'sleep') as fake_thread, \
            mock.patch.object(jsonutils, 'loads',
                              return_value=fake_resp) as mock_loads, \
            mock.patch.object(self.fakesocket,
                              'recv',
                              return_value=fake_resp) as mock_sock_rcv, \
            mock.patch.object(self.fakesocket,
                              'send') as mock_sock_send:
            self.l2gw_ovsdb_conn._echo_response(self.fake_ip)
            self.assertTrue(fake_thread.called)
            self.assertTrue(mock_sock_rcv.called)
            mock_loads.assert_called_with(fake_resp)
            self.assertTrue(self.l2gw_ovsdb_conn.check_c_sock)
            self.assertTrue(mock_sock_send.called)

    def test_common_sock_rcv_thread_none(self):
        with mock.patch.object(base_connection.BaseConnection,
                               '_echo_response') as mock_resp, \
            mock.patch.object(eventlet.greenthread, 'sleep'
                              ) as green_thrd_sleep, \
            mock.patch.object(self.fakesocket,
                              'recv', return_value=None) as mock_rcv, \
            mock.patch.object(base_connection.BaseConnection,
                              'disconnect') as mock_disconnect:
            self.l2gw_ovsdb_conn.check_c_sock = True
            self.l2gw_ovsdb_conn.read_on = True
            self.l2gw_ovsdb_conn._common_sock_rcv_thread(self.fake_ip)
            self.assertTrue(mock_resp.called)
            self.assertTrue(green_thrd_sleep.called)
            self.assertTrue(mock_rcv.called)
            self.assertTrue(mock_disconnect.called)
            self.assertFalse(self.l2gw_ovsdb_conn.connected)
            self.assertFalse(self.l2gw_ovsdb_conn.read_on)

    def test_disconnect_with_enable_manager(self):
        fake_ip = 'fake_ip'
        self.l2gw_ovsdb_conn.ovsdb_fd_states = {fake_ip: 'fake_status'}
        self.l2gw_ovsdb_conn.ovsdb_conn_list = [fake_ip]
        with mock.patch.object(self.fakesocket,
                               'close') as (mock_close):
            self.l2gw_ovsdb_conn.disconnect(fake_ip)
            self.assertTrue(mock_close.called)
            self.assertNotIn(fake_ip, self.l2gw_ovsdb_conn.ovsdb_fd_states)
            self.assertNotIn(fake_ip, self.l2gw_ovsdb_conn.ovsdb_conn_list)

    def test_get_ovsdb_ip_mapping(self):
        expected_ovsdb_ip_mapping = {'10.10.10.10': 'ovsdb1'}
        cfg.CONF.set_override('ovsdb_hosts',
                              'ovsdb1:10.10.10.10:6632',
                              'ovsdb')
        return_mapping = self.l2gw_ovsdb_conn._get_ovsdb_ip_mapping()
        self.assertEqual(expected_ovsdb_ip_mapping, return_mapping)

    def test_is_ssl_configured(self):
        self.l2gw_ovsdb_conn.ip_ovsdb_mapping = {'10.10.10.10': 'ovsdb1'}
        cfg.CONF.set_override('l2_gw_agent_priv_key_base_path',
                              '/home',
                              'ovsdb')
        cfg.CONF.set_override('l2_gw_agent_cert_base_path',
                              '/home',
                              'ovsdb')
        cfg.CONF.set_override('l2_gw_agent_ca_cert_base_path',
                              '/home',
                              'ovsdb')
        with mock.patch.object(os.path, 'isfile',
                               return_value=True) as mock_isfile, \
                mock.patch.object(base_connection.LOG,
                                  'error') as mock_error, \
                mock.patch.object(ssl, 'wrap_socket') as mock_wrap_sock:
            self.l2gw_ovsdb_conn._is_ssl_configured('10.10.10.10',
                                                    self.mock_sock)
            self.assertTrue(mock_isfile.called)
            self.assertTrue(mock_wrap_sock.called)
            self.assertFalse(mock_error.called)
            mock_wrap_sock.assert_called_with(
                self.mock_sock,
                server_side=True,
                keyfile='/home/ovsdb1.key',
                certfile='/home/ovsdb1.cert',
                ssl_version=ssl.PROTOCOL_SSLv23,
                ca_certs='/home/ovsdb1.ca_cert')

    def test_is_ssl_configured_for_certs_not_found(self):
        self.l2gw_ovsdb_conn.ip_ovsdb_mapping = {'10.10.10.10': 'ovsdb1'}
        cfg.CONF.set_override('l2_gw_agent_priv_key_base_path',
                              '/home/',
                              'ovsdb')
        cfg.CONF.set_override('l2_gw_agent_cert_base_path',
                              '/home/',
                              'ovsdb')
        cfg.CONF.set_override('l2_gw_agent_ca_cert_base_path',
                              '/home/',
                              'ovsdb')
        with mock.patch.object(os.path, 'isfile',
                               return_value=False) as mock_isfile, \
                mock.patch.object(base_connection.LOG, 'error'
                                  ) as mock_error, \
                mock.patch.object(ssl, 'wrap_socket') as mock_wrap_sock:
            self.l2gw_ovsdb_conn._is_ssl_configured('10.10.10.10',
                                                    self.mock_sock)
            self.assertTrue(mock_isfile.called)
            self.assertFalse(mock_wrap_sock.called)
            self.assertTrue(mock_error.called)



# Copyright (c) 2015 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron_lib.db import model_base
import sqlalchemy as sa


class PhysicalLocators(model_base.BASEV2):
    __tablename__ = 'physical_locators'
    uuid = sa.Column(sa.String(36), nullable=False, primary_key=True)
    dst_ip = sa.Column(sa.String(64), nullable=True)
    ovsdb_identifier = sa.Column(sa.String(64), nullable=False,
                                 primary_key=True)


class PhysicalSwitches(model_base.BASEV2):
    __tablename__ = 'physical_switches'
    uuid = sa.Column(sa.String(36), nullable=False, primary_key=True)
    name = sa.Column(sa.String(255), nullable=True)
    tunnel_ip = sa.Column(sa.String(64), nullable=True)
    ovsdb_identifier = sa.Column(sa.String(64), nullable=False,
                                 primary_key=True)
    switch_fault_status = sa.Column(sa.String(length=32), nullable=True)


class PhysicalPorts(model_base.BASEV2):
    __tablename__ = 'physical_ports'
    uuid = sa.Column(sa.String(36), nullable=False, primary_key=True)
    name = sa.Column(sa.String(255), nullable=True)
    physical_switch_id = sa.Column(sa.String(36), nullable=True)
    ovsdb_identifier = sa.Column(sa.String(64), nullable=False,
                                 primary_key=True)
    port_fault_status = sa.Column(sa.String(length=32), nullable=True)


class LogicalSwitches(model_base.BASEV2):
    __tablename__ = 'logical_switches'
    uuid = sa.Column(sa.String(36), nullable=False, primary_key=True)
    name = sa.Column(sa.String(255), nullable=True)
    key = sa.Column(sa.Integer, nullable=True)
    ovsdb_identifier = sa.Column(sa.String(64), nullable=False,
                                 primary_key=True)


class UcastMacsLocals(model_base.BASEV2):
    __tablename__ = 'ucast_macs_locals'
    uuid = sa.Column(sa.String(36), nullable=False, primary_key=True)
    mac = sa.Column(sa.String(32), nullable=True)
    logical_switch_id = sa.Column(sa.String(36), nullable=True)
    physical_locator_id = sa.Column(sa.String(36), nullable=True)
    ip_address = sa.Column(sa.String(64), nullable=True)
    ovsdb_identifier = sa.Column(sa.String(64), nullable=False,
                                 primary_key=True)


class UcastMacsRemotes(model_base.BASEV2):
    __tablename__ = 'ucast_macs_remotes'
    uuid = sa.Column(sa.String(36), nullable=False, primary_key=True)
    mac = sa.Column(sa.String(32), nullable=True)
    logical_switch_id = sa.Column(sa.String(36), nullable=True)
    physical_locator_id = sa.Column(sa.String(36), nullable=True)
    ip_address = sa.Column(sa.String(64), nullable=True)
    ovsdb_identifier = sa.Column(sa.String(64), nullable=False,
                                 primary_key=True)


class VlanBindings(model_base.BASEV2):
    __tablename__ = 'vlan_bindings'
    port_uuid = sa.Column(sa.String(36), nullable=False, primary_key=True)
    vlan = sa.Column(sa.Integer, nullable=False, primary_key=True)
    logical_switch_uuid = sa.Column(sa.String(36), nullable=False,
                                    primary_key=True)
    ovsdb_identifier = sa.Column(sa.String(64), nullable=False,
                                 primary_key=True)


class PendingUcastMacsRemote(model_base.BASEV2, model_base.HasId):
    __tablename__ = 'pending_ucast_macs_remotes'
    uuid = sa.Column(sa.String(36), nullable=True)
    mac = sa.Column(sa.String(32), nullable=False)
    logical_switch_uuid = sa.Column(sa.String(36), nullable=False)
    locator_uuid = sa.Column(sa.String(36), nullable=True)
    dst_ip = sa.Column(sa.String(64))
    vm_ip = sa.Column(sa.String(64))
    ovsdb_identifier = sa.Column(sa.String(64), nullable=False)
    operation = sa.Column(sa.String(8), nullable=False)
    timestamp = sa.Column(sa.DateTime, nullable=False)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc


class L2gwDriverBase(object, metaclass=abc.ABCMeta):

    def __init__(self, service_plugin, validator=None):
        self.service_plugin = service_plugin

    @property
    def service_type(self):
        pass

    @abc.abstractmethod
    def add_port_mac(self, context, port_dict):
        pass

    @abc.abstractmethod
    def delete_port_mac(self, context, port):
        pass

    @abc.abstractmethod
    def create_l2_gateway(self, context, l2_gateway):
        pass

    @abc.abstractmethod
    def update_l2_gateway(self, context, l2_gateway_id, l2_gateway):
        pass

    @abc.abstractmethod
    def delete_l2_gateway(self, context, l2_gateway_id):
        pass

    @abc.abstractmethod
    def create_l2_gateway_connection(self, context, l2_gateway_connection):
        pass

    @abc.abstractmethod
    def delete_l2_gateway_connection(self, context, l2_gateway_connection_id):
        pass

    @abc.abstractmethod
    def create_l2_gateway_precommit(self, context, l2_gateway):
        pass

    @abc.abstractmethod
    def update_l2_gateway_precommit(self, context, l2_gateway):
        pass

    @abc.abstractmethod
    def delete_l2_gateway_precommit(self, context, l2_gateway_id):
        pass

    @abc.abstractmethod
    def create_l2_gateway_connection_precommit(self, context,
                                               l2_gateway_connection):
        pass

    @abc.abstractmethod
    def delete_l2_gateway_connection_precommit(self, context,
                                               l2_gateway_connection_id):
        pass

    @abc.abstractmethod
    def create_l2_gateway_postcommit(self, context, l2_gateway):
        pass

    @abc.abstractmethod
    def delete_l2_gateway_postcommit(self, context, l2_gateway_id):
        pass

    @abc.abstractmethod
    def update_l2_gateway_postcommit(self, context, l2_gateway):
        pass

    @abc.abstractmethod
    def create_l2_gateway_connection_postcommit(self, context,
                                                l2_gateway_connection):
        pass

    @abc.abstractmethod
    def delete_l2_gateway_connection_postcommit(self, context,
                                                l2_gateway_connection_id):
        pass


class L2gwDriver(L2gwDriverBase, metaclass=abc.ABCMeta):

    def __init__(self, service_plugin, validator=None):
        super(L2gwDriver, self).__init__(service_plugin)

    @property
    def service_type(self):
        pass

    def add_port_mac(self, context, port_dict):
        pass

    def delete_port_mac(self, context, port):
        pass

    def create_l2_gateway(self, context, l2_gateway):
        pass

    def update_l2_gateway(self, context, l2_gateway_id, l2_gateway):
        pass

    def delete_l2_gateway(self, context, l2_gateway_id):
        pass

    def create_l2_gateway_connection(self, context, l2_gateway_connection):
        pass

    def delete_l2_gateway_connection(self, context, l2_gateway_connection_id):
        pass

    def create_l2_gateway_precommit(self, context, l2_gateway):
        pass

    def update_l2_gateway_precommit(self, context, l2_gateway):
        pass

    def delete_l2_gateway_precommit(self, context, l2_gateway_id):
        pass

    def create_l2_gateway_connection_precommit(self, context,
                                               l2_gateway_connection):
        pass

    def delete_l2_gateway_connection_precommit(self, context,
                                               l2_gateway_connection_id):
        pass

    def create_l2_gateway_postcommit(self, context, l2_gateway):
        pass

    def delete_l2_gateway_postcommit(self, context, l2_gateway_id):
        pass

    def update_l2_gateway_postcommit(self, context, l2_gateway):
        pass

    def create_l2_gateway_connection_postcommit(self, context,
                                                l2_gateway_connection):
        pass

    def delete_l2_gateway_connection_postcommit(self, context,
                                                l2_gateway_connection_id):
        pass



# All Rights Reserved 2018
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from oslo_utils import uuidutils

from networking_l2gw.l2gatewayclient.osc import l2gw as osc_l2gw
from networking_l2gw.l2gatewayclient.osc import l2gw_connection as \
    osc_l2gw_conn


class FakeL2GW(object):

    @staticmethod
    def create_l2gw(num_dev=1, num_if=1, attrs=None):
        """Create one fake L2 Gateway."""
        attrs = attrs or {}
        interfaces = [{'name': 'interface' +
                               uuidutils.generate_uuid(dashed=False)} for
                      iface in range(num_if)]
        devices = [{'device_name': 'device' +
                                   uuidutils.generate_uuid(dashed=False),
                    'interfaces': interfaces} for dev in range(num_dev)]
        l2gw_attrs = {
            'id': uuidutils.generate_uuid(),
            'name': 'test-l2gw' + uuidutils.generate_uuid(dashed=False),
            'tenant_id': uuidutils.generate_uuid(),
            'devices': devices
        }

        l2gw_attrs.update(attrs)
        return copy.deepcopy(l2gw_attrs)

    @staticmethod
    def create_l2gws(attrs=None, count=1):
        """Create multiple fake L2 Gateways."""

        l2gws = []
        for i in range(0, count):
            if attrs is None:
                attrs = {'id': 'fake_id%d' % i}
            elif getattr(attrs, 'id', None) is None:
                attrs['id'] = 'fake_id%d' % i
            l2gws.append(FakeL2GW.create_l2gw(attrs=attrs))

        return {osc_l2gw.L2_GATEWAYS: l2gws}


class FakeL2GWConnection(object):

    @staticmethod
    def create_l2gw_connection(attrs=None):
        """Create a fake l2gw connection."""

        attrs = attrs or {}
        l2gw_connection_attrs = {
            'network_id': uuidutils.generate_uuid(),
            'l2_gateway_id': uuidutils.generate_uuid(),
            'segmentation_id': '42',
            'tenant_id': uuidutils.generate_uuid(),
            'id': uuidutils.generate_uuid()
        }

        l2gw_connection_attrs.update(attrs)
        return copy.deepcopy(l2gw_connection_attrs)

    @staticmethod
    def create_l2gw_connections(attrs=None, count=1):
        l2gw_connections = []

        for i in range(0, count):
            if attrs is None:
                attrs = {'id': 'fake_id%d' % i}
            elif getattr(attrs, 'id', None) is None:
                attrs['id'] = 'fake_id%d' % i
            l2gw_connections.append(FakeL2GWConnection.create_l2gw_connection(
                attrs=attrs))

        return {osc_l2gw_conn.L2_GATEWAY_CONNECTIONS: l2gw_connections}



# Copyright (c) 2016 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron.plugins.ml2.drivers.l2pop import rpc as l2pop_rpc
from neutron.plugins.ml2 import managers
from neutron.plugins.ml2 import rpc as rpc
from neutron_lib.agent import topics


class Tunnel_Calls(object):
    """Common tunnel calls for L2 agent."""
    def __init__(self):
        self._construct_rpc_stuff()

    def _construct_rpc_stuff(self):
        self.notifier = rpc.AgentNotifierApi(topics.AGENT)
        self.type_manager = managers.TypeManager()
        self.tunnel_rpc_obj = rpc.RpcCallbacks(self.notifier,
                                               self.type_manager)

    def trigger_tunnel_sync(self, context, tunnel_ip):
        """Sends tunnel sync RPC message to the neutron

        L2 agent.
        """
        tunnel_dict = {'tunnel_ip': tunnel_ip,
                       'tunnel_type': 'vxlan'}
        self.tunnel_rpc_obj.tunnel_sync(context,
                                        **tunnel_dict)

    def trigger_l2pop_sync(self, context, other_fdb_entries):
        """Sends L2pop ADD RPC message to the neutron L2 agent."""
        l2pop_rpc.L2populationAgentNotifyAPI().add_fdb_entries(
            context, other_fdb_entries)

    def trigger_l2pop_delete(self, context, other_fdb_entries, host=None):
        """Sends L2pop DELETE RPC message to the neutron L2 agent."""
        l2pop_rpc.L2populationAgentNotifyAPI().remove_fdb_entries(
            context, other_fdb_entries, host)



# Copyright (c) 2016 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class OvsdbObject(object):
    def __init__(self, uuid):
        self.uuid = uuid


class LogicalSwitch(OvsdbObject):
    def __init__(self, uuid, name, description, tunnel_key):
        super(LogicalSwitch, self).__init__(uuid)
        self.name = name
        self.description = description
        self.tunnel_key = tunnel_key


class PhysicalLocatorSet(OvsdbObject):
    def __init__(self, uuid, locator_uuid_list):
        super(PhysicalLocatorSet, self).__init__(uuid)
        self.locator_uuid_list = locator_uuid_list


class PhysicalLocator(OvsdbObject):
    def __init__(self, uuid,
                 dst_ip, tunnel_key=None,
                 encapsulation_type='vxlan_over_ipv4'):
        super(PhysicalLocator, self).__init__(uuid)
        self.dst_ip = dst_ip
        self.encapsulation_type = encapsulation_type
        self.tunnel_key = tunnel_key


class UcastMacs(OvsdbObject):
    def __init__(self, uuid, mac, ipaddr, logical_switch_uuid, locator_uuid):
        super(UcastMacs, self).__init__(uuid)
        self.mac = mac
        self.ipaddr = ipaddr
        self.logical_switch_uuid = logical_switch_uuid
        self.locator_uuid = locator_uuid


class McastMacs(OvsdbObject):
    def __init__(self, uuid, mac, dst_ip, logical_switch_uuid,
                 locator_set__uuid):
        super(McastMacs, self).__init__(uuid)
        self.mac = mac
        self.dst_ip = dst_ip
        self.logical_switch_uuid = logical_switch_uuid
        self.locator_set__uuid = locator_set__uuid


class PhysicalPort(OvsdbObject):
    def __init__(self, uuid, name, description, vlan_bindings_dict):
        super(PhysicalPort, self).__init__(uuid)
        self.name = name
        self.description = description
        self.vlan_bindings_dict = vlan_bindings_dict
